/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_GPGME_DATA
#define HGL_COMMON_GPGME_DATA

#include <deque>
#include <cerrno>

#include <stdint.h>
#include <sys/types.h>

#include "gpgmeexception.h"
#include "l10n.h"

extern template class std::deque<uint8_t>;

namespace HGL {

namespace Common {

namespace GPGME {

class _INTERNAL Data {
	DISALLOW_COPY_AND_ASSIGN(Data)
public:
	_EXPORT Data() throw(GPGMEException);

	template<class Type>
	explicit Data(const Type &d) throw(GPGMEException);

	_EXPORT ~Data() throw();

#if ENABLE_SIGNING
	inline operator gpgme_data_t() const throw() {
		return m_data;
	}
#endif

	_EXPORT off_t size() const throw(GPGMEException);

	template<class OIter>
	void read(OIter iter) const throw(GPGMEException);

private:
#if ENABLE_SIGNING
	gpgme_data_t m_data;
#endif
	char *m_buffer;
};

template<>
#if ENABLE_SIGNING
Data::Data(const std::string &s) throw(GPGMEException) : m_data(NULL), m_buffer(NULL) {

	gpgme_error_t err = gpgme_data_new_from_mem(&m_data, s.c_str(), s.length(), 0);

	if(unlikely(err != GPG_ERR_NO_ERROR)) {
		throw GPGMEException(_("Creating GPGME data failed"), err);
	}

#else
Data::Data(const std::string &) throw(GPGMEException) : m_buffer(NULL) {
#endif
}

template<class Type>
#if ENABLE_SIGNING
Data::Data(const Type &d) throw(GPGMEException) : m_data(NULL), m_buffer(NULL) {

	size_t size = sizeof(typename Type::value_type) * d.size();

	m_buffer = new(std::nothrow) char[size];

	if(likely(m_buffer != 0L)) {

		std::copy(d.begin(), d.end(), m_buffer);

		gpgme_error_t err = gpgme_data_new_from_mem(&m_data, m_buffer, size, 0);

		if(unlikely(err != GPG_ERR_NO_ERROR)) {
			delete [] m_buffer;
			throw GPGMEException(_("Creating GPGME data failed"), err);
		}

	} else {
		throw GPGMEException(_("Creating GPGME data failed"), GPG_ERR_ENOMEM);
	}

#else
Data::Data(const Type &) throw(GPGMEException) : m_buffer(NULL) {
#endif
}

template<class OIter>
void Data::read(OIter iter) const throw(GPGMEException) {
#if ENABLE_SIGNING

	if(unlikely(gpgme_data_seek(m_data, 0, SEEK_SET) == -1)) {
		throw GPGMEException(_("Reading data failed"), gpgme_err_code_from_errno(errno));
	}

	off_t pos = 0;
	size_t eSize = sizeof(typename std::iterator_traits<OIter>::value_type);
	typename std::iterator_traits<OIter>::value_type entity;

	while(pos <= size()) {

		ssize_t rd;

		if((rd = gpgme_data_read(m_data, &entity, eSize)) == eSize) {
			pos += rd;
			*iter++ = entity;
		} else if(unlikely(rd)) {
			throw GPGMEException(_("Reading data failed"), gpgme_err_code_from_errno(errno));
		} else if(!rd) {
			break;
		}
	}

#endif
}

}

}

}

#endif /* HGL_COMMON_GPGME_DATA */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
