/*
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef HGL_COMMON_IAPPLICATION_H
#define HGL_COMMON_IAPPLICATION_H

#include <string>

#include "linkercontrol.h"

namespace HGL {

namespace Common {

class IApplicationObserver;

/**
 * @ingroup util
 * @interface IApplication
 * @brief Interface to a typical HGL Suite application
 *
 * A typical HGL Suite application should implement this interface, resp. derive from
 * HGL::Common::Application.
 *
 * Since version @c 0.5.25 it is tried to register with a desktop notification daemon.
 * If there is such one and the config value @c enable-notifications is @c true a
 * desktop notification can be emitted by calling @c notifyInfo, @c notifyWarn or
 * @c notifyError else if the config value @c enable-notifications is @c true a console
 * log to the respective logger is emitted.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _INTERNAL IApplication {
	DISALLOW_COPY_AND_ASSIGN(IApplication)
public:
	virtual ~IApplication() {}

	/**
	 * @brief Indicates if the application got a termination signal and should prepare to shutdown
	 *
	 * @return <code>true</code> if the application shut prepare to shut down,
	 * <code>false</code> otherwise
	 */
	virtual bool hasTerminated() const = 0;

	/**
	 * @brief Registers an IApplicationObserver
	 *
	 * @param iao an IApplicationObserver to register
	 */
	virtual void registerApplicationObserver(const IApplicationObserver *const iao) const = 0;

	/**
	 * @brief Unregisters an IApplicationObserver
	 *
	 * @param iao an IApplicationObserver to unregister
	 */
	virtual void unregisterApplicationObserver(const IApplicationObserver *const iao) const = 0;

	/**
	 * @brief Behaviour options to be passed to the @c notifyXXX functions
	 * @since 0.5.25
	 */
	typedef enum {
		DEFAULT, ///< a default notification, which expires automatically
		UPDATE,  ///< an UPDATE notification, which expires never expires until @c CLOSEd
		CLOSE    ///< closes the last emitted notification
	} NOTIFYOPT;

	/**
	  * @brief Checks if notifications are available and enabled
	  *
	  * @return @c true if notifications are available, @c false otherwise
	  *
	  * @since 0.5.25
	  */
	virtual bool notificationsAvailable() const = 0;

	/**
	 * @brief Emits an information desktop notification if possible
	 *
	 * @param msg the message to display
	 * @param opt behaviour option
	 * @param summary an optional summary
	 *
	 * @since 0.5.25
	 */
	virtual void notifyInfo(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							const std::string &summary = std::string()) const = 0;

	/**
	 * @brief Emits a warning desktop notification if possible
	 *
	 * @param msg the message to display
	 * @param opt behaviour option
	 * @param summary an optional summary
	 *
	 * @since 0.5.25
	 */
	virtual void notifyWarn(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							const std::string &summary = std::string()) const = 0;

	/**
	 * @brief Emits an error desktop notification if possible
	 *
	 * @param msg the message to display
	 * @param opt behaviour option
	 * @param summary an optional summary
	 *
	 * @since 0.5.25
	 */
	virtual void notifyError(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							 const std::string &summary = std::string()) const = 0;

	/**
	 * @brief Emits an fatal error desktop notification if possible
	 *
	 * @param msg the message to display
	 * @param opt behaviour option
	 * @param summary an optional summary
	 *
	 * @since 0.5.25
	 */
	virtual void notifyFatal(const std::string &msg, NOTIFYOPT opt = DEFAULT,
							 const std::string &summary = std::string()) const = 0;

protected:
	inline IApplication() {}
};

}

}

#endif /* HGL_COMMON_IAPPLICATION_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
