/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_IMAGEOUTPUT_ICANVAS_H
#define HGL_IMAGEOUTPUT_ICANVAS_H

#ifndef M_LN2l
#define M_LN2l 0.693147180559945309417232121458176568L
#endif

/**
 * @file icanvas.h
 * @brief Macros and interfaces to the canvas
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */

/**
 * @ingroup ioplugins
 * @def COLORSPACEINDEX
 * @hideinitializer
 * @brief Returns the index (position) of the color within the colorspace.
 *
 * Use this macro if you need the index (position) of a given color to reference that color
 *
 * @note A non-existing color will return a value beyond the size of the colorspace
 *
 * @param colorspace the colorspace
 * @param index the color index
 *
 * @return the index (position) of the given color
 *
 * @relates HGL::ImageOutput::ICanvas
 *
 * @since 0.4.49
 **/
#define COLORSPACEINDEX(colorspace, index) \
	std::distance(colorspace.begin(), colorspace.find((index)))

/**
 * @ingroup ioplugins
 * @def RGBA2INDEX
 * @hideinitializer
 * @brief Converts @c red, @c green, @c blue and @c alpha to an unsigned 32 bit integer
 *
 * @param r unsigned 8 bit red value
 * @param g unsigned 8 bit green value
 * @param b unsigned 8 bit blue value
 * @param a unsigned 8 bit alpha value
 *
 * @return an unsigned 32 bit integer
 *
 * @relates HGL::ImageOutput::ICanvas::RGBA
 *
 * @since 0.5.32
 **/
#define RGBA2INDEX(r, g, b, a) (static_cast<uint32_t>((a) | ((b) << static_cast<uint8_t>(0x08u)) | \
								((g) << static_cast<uint8_t>(0x10u)) | \
								((r) << static_cast<uint8_t>(0x18u))))

/**
 * @ingroup ioplugins
 * @def MAKETRANS
 * @hideinitializer
 * @brief Sets the alpha channel to @c 0x00
 *
 * @param idx an unsigned 32 bit integer
 *
 * @return the new color index
 *
 * @relates HGL::ImageOutput::ICanvas::RGBA
 *
 * @since 0.5.40
 **/
#define MAKETRANS(idx) (uint32_t(idx) & ~0xFFFFFF)

/**
 * @ingroup ioplugins
 * @def MAKEOPAQUE
 * @hideinitializer
 * @brief Sets the alpha channel to @c 0xFF
 *
 * @param idx an unsigned 32 bit integer
 *
 * @return the new color index
 *
 * @relates HGL::ImageOutput::ICanvas::RGBA
 *
 * @since 0.5.40
 **/
#define MAKEOPAQUE(idx) (uint32_t(idx) | 0xFF)

/**
 * @ingroup ioplugins
 * @def CSDEPTH
 * @hideinitializer
 * @brief Determines the depth of the @c COLORSPACE in bits
 *
 * @param cs a @c ICanvas::COLORSPACE
 *
 * @return std::size_t the depth in bits
 *
 * @since 0.5.36
 **/
#define CSDEPTH(cs) std::size_t(std::ceil(std::log((cs).size())/M_LN2l))

#include <set>
#include <vector>
#include <string>

#include "eff_map.h"
#include "q24format.h"
#include "imageoutputexception.h"

namespace HGL {

namespace Type {
class Line;
class IPoint;
class IProject;
}

#if INTERPRETER
namespace Interpreter {

namespace Render {
class Canvas;
class CanvasImpl;
}

}
#endif

namespace ImageOutput {

class IPostProcessor;

typedef struct _INTERNAL _bucket {
	inline _bucket(int y = 0, float x = 0.0f, float i = 0.0f) throw() : inc(i), x_min(x), y_max(y)
	{}

	inline bool operator<(const _bucket &b) const throw() {
		return ((y_max == b.y_max) ? y_max <= b.y_max && x_min < b.x_min : y_max <= b.y_max);
	}

	inline _bucket &operator=(_bucket o) {
		swap(*this, o);
		return *this;
	}

	friend inline void swap(_bucket &l, _bucket &r) {
		using std::swap;

		if(&l != &r) {
			swap(l.x_min, r.x_min);
			swap(l.inc, r.inc);

			l.y_max ^= r.y_max;
			r.y_max ^= l.y_max;
			l.y_max ^= r.y_max;
		}
	}

	float 			inc;
	float 			x_min;
	int_fast16_t	y_max;
} BUCKET;

typedef std::multiset<BUCKET> BUCKETSET;
typedef std::map<int_fast16_t, BUCKETSET, std::greater<int_fast16_t> > EDGETABLE;

typedef std::vector < std::pair < std::pair<HGL::Type::Line *, std::vector<HGL::Type::IPoint *> >,
		bool > > LINELIST;

/**
 * @ingroup ioplugins
 * @interface ICanvas
 * @brief Interface to access @c Canvas information and methods
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/
class ICanvas {
	ICanvas(const ICanvas &);
	ICanvas &operator=(const ICanvas &);
#if INTERPRETER
	friend class HGL::Interpreter::Render::CanvasImpl;
#endif

	class PenBase;

public:
	typedef class PenBase PEN;

	/**
	 * @ingroup ioplugins
	 * @brief This structure holds the calculated transformation info
	 * @since 0.5.2
	 **/
	typedef struct __attribute__((__packed__)) _transformationInfo {
		inline _transformationInfo() : bbox_x(0), bbox_w(0),
			bbox_y(0), bbox_h(0), centerOffsetX(0.0f), centerOffsetY(0.0f),
			scaleFactorX(1.0f), scaleFactorY(1.0f) {}

	/**
	 * @brief the x position of the boundingbox
	 */
	float bbox_x;

	/**
	 * @brief the width of the boundingbox
	 */
	float bbox_w;

	/**
	 * @brief the y position of the boundingbox
	 */
	float bbox_y;

	/**
	 * @brief the height of the boundingbox
	 */
	float bbox_h;

	/**
	 * @brief the x offset to center the image in the canvas
	 */
	float centerOffsetX;

	/**
	 * @brief the y offset to center the image in the canvas
	 */
	float centerOffsetY;

	/**
	 * @brief the x scale factor
	 */
	float scaleFactorX;

	/**
	 * @brief the y scale factor
	 */
	float scaleFactorY;
	} TRANSFORMATIONINFO;

	struct RGBA;

	/**
	 * @brief Destructor
	 *
	 **/
	virtual ~ICanvas() = 0;

	/**
	 * @name Dimensions
	 * @{
	 */

	/**
	 * @brief Gets the width of the canvas
	 *
	 * @return the width in pixels
	 **/
	virtual uint16_t getWidth() const = 0;

	/**
	 * @brief Gets the height of the canvas
	 *
	 * @return the height in pixels
	 **/
	virtual uint16_t getHeight() const = 0;

	/**
	 * @brief Determines if explicit scale should be enabled
	 *
	 * Explicit scale is enabled if either the 'w' or 'h' option or both are given on the
	 * command line. If not the default values for width and height are taken and this
	 * function returns <code>false</code>.
	 *
	 * @return bool <code>true</code> if the output image should get scaled explicitely,
	 * <code>false</code> otherwise
	 * @since 0.5.2
	 */
	virtual bool doExplicitScale() const = 0;

	//@}

	/**
	 * @name Data access
	 * @{
	 */

	/**
	 * @brief Gets the complete processed project
	 *
	 * @return the complete processed project
	 * @since 0.5.21
	 **/
	virtual HGL::Type::IProject *getProject() const = 0;

	typedef std::vector<RGBA> CANVASROW;

	/**
	 * @brief Gets a row of the canvas as vector of colors
	 *
	 * @param row number of the row
	 * @return an vector of colors of getWidth() size
	 **/
	virtual const CANVASROW getRow(uint16_t row) const = 0;

	//@}

	/**
	 * @name Post processing
	 * @{
	 */

	/**
	 * @brief Adds a post processor
	 *
	 * @pre add the post processor before the first call to getRow in order to activate it
	 *
	 * @param postProc the post processor to add
	 *
	 * @since 0.5.37
	 **/
	virtual void addPostProcessor(const IPostProcessor *postProc) const = 0;

	//@}

	/**
	 * @name Color management
	 * @{
	 */

private:
	class _histogram;

public:
	/**
	 * @brief A colorspace set
	 * @since 0.5.5
	 */
	typedef std::set<uint32_t> COLORSPACE;

	/**
	 * @brief A histogram
	 * @since 0.5.37
	 */
	typedef _histogram HISTOGRAM;

	/**
	 * @brief Gets a set with all unique colors in the current ICanvas
	 *
	 * @param opaque if @c true then set alpha value to opaque (0xFF)
	 *
	 * @return a set with unique color indices
	 * @see RGBA::getColorIndex() const
	 * @see COLORSPACEINDEX(colorspace, index)
	 * @since 0.4.49
	 *
	 **/
	virtual const COLORSPACE getColorSpace(bool opaque = false) const = 0;

	/**
	 * @brief Gets a histogram of all colors in the current ICanvas
	 *
	 * @param opaque if @c true then set alpha value to opaque (0xFF)
	 *
	 * @return a histogram of colors
	 * @since 0.4.37
	 *
	 **/
	virtual const HISTOGRAM getHistogram(bool opaque = false) const = 0;

	/**
	 * @brief Checks if the given position is the unaltered background
	 *
	 * @param x the x-coordinate
	 * @param y the y-coordinate
	 * @return @c true if the background is unaltered, @c false otherwise
	 * @since 0.5.14
	 */
	virtual bool isBackground(uint16_t x, uint16_t y) const = 0;

	/**
	 * @brief Gets the background color of the ICanvas
	 *
	 * @return the background color of the ICanvas
	 * @since 0.5.14
	 */
	virtual const RGBA &getBackgroundColor() const = 0;

	/**
	 * @brief Checks if the image is a greyscale image
	 *
	 * @return @ctrue if the image is a greyscale image, @c false otherwise
	 * @since 0.5.38
	 */
	virtual bool isGreyScaled() const = 0;

	//@}

protected:
	/**
	 * @brief Constructor
	 *
	 **/
	inline ICanvas() {}

public:
	static RGBA WHITE;
	static RGBA BLACK;
	_EXPORT static RGBA TRANS;
};

/**
 * @ingroup ioplugins
 * @brief Structure representing a color
 *
 * Includes the alpha channel
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
struct _EXPORT ICanvas::RGBA {
	/**
	 * @brief Constructor
	 *
	 * Initial is color white with no opacity (resp. full transparent)
	 **/
	RGBA() throw();

	/**
	 * @brief Constructor
	 *
	 * @param hexStr a string in the form rrggbbaa
	 **/
	explicit RGBA(const std::string &hexStr) throw();

	/**
	 * @brief Constructor
	 *
	 * @param r the red value between 0 and 255
	 * @param g the green value between 0 and 255
	 * @param b the blue value between 0 and 255
	 * @param a the alpha value between 0 and 255
	 **/
	inline RGBA(uint8_t r, uint8_t g, uint8_t b, uint8_t a = 0xFF) throw() : red(r), green(g),
		blue(b), alpha(a) {}

	/**
	 * @brief Constructor
	 *
	 * Constructs an <code>RGBA</code> instance from a 32 bit integer.
	 *
	 * The integer should have a hex form like 0xrrggbbaa.
	 *
	 * @see RGBA(const std::string &hexStr)
	 * @param index the color index
	 * @since 0.4.49
	 **/
	inline RGBA(uint32_t index) throw() : red(static_cast<uint8_t>(index >> 0x18u)),
		green(static_cast<uint8_t>(index >> 0x10u)), blue(static_cast<uint8_t>(index >> 0x08u)),
		alpha(static_cast<uint8_t>(index)) {}

	/**
	 * @brief Destructor
	 *
	 **/
	virtual ~RGBA() throw();

	friend _EXPORT void swap(RGBA &l, RGBA &r);

	RGBA &operator=(uint32_t o);

	inline RGBA &operator+=(uint32_t o) {
		return (*this = (RGBA2INDEX(red, green, blue, alpha) + o));
	}

	inline RGBA &operator=(const RGBA &o) {
		if(this != &o) {
			red = o.red;
			green = o.green;
			blue = o.blue;
			alpha = o.alpha;
		}

		return *this;
	}

	inline RGBA operator-() const throw() {
		return RGBA(0xFF - red, 0xFF - green, 0xFF - blue, 0xFF - alpha);
	}

	inline bool operator==(const RGBA &r) const throw() {
		return red == r.red && green == r.green && blue == r.blue && alpha == r.alpha;
	}

	inline bool operator<(const RGBA &r) const throw() {
		return RGBA2INDEX(red, green, blue, alpha) <
			   RGBA2INDEX(r.red, r.green, r.blue, r.alpha);
	}

	/**
	 * @brief Gets the color index
	 *
	 * @return the color index
	 * @since 0.5.40
	 **/
	inline operator uint32_t() const throw() {
		return RGBA2INDEX(red, green, blue, alpha);
	}

	/**
	 * @brief the red value between 0 and 255
	 **/
	uint8_t red;

	/**
	 * @brief the green value between 0 and 255
	 **/
	uint8_t green;

	/**
	 * @brief the blue value between 0 and 255
	 **/
	uint8_t blue;

	/**
	 * @brief the alpha value between 0 and 255
	 **/
	uint8_t alpha;
};

#if INTERPRETER
class ICanvas::_histogram {
	_histogram &operator=(const _histogram &);

	friend class HGL::Interpreter::Render::Canvas;
	friend class HGL::Interpreter::Render::CanvasImpl;

	typedef std::map<COLORSPACE::value_type, HGL::Common::Q24> HISTOMAP;
public:
	typedef std::multimap < HISTOMAP::mapped_type, HISTOMAP::key_type,
			std::less<HISTOMAP::mapped_type> > FLIPHIST;

	COLORSPACE colorSpace() const;
	HISTOMAP::size_type size() const _PURE;
	FLIPHIST flip() const;

private:
	_histogram();

private:
	HISTOMAP m_data;
};
#endif

class ICanvas::PenBase {
	struct PenData {
		PenData &operator=(const PenData &);

		PenData(const RGBA &c = RGBA(0xFFFFFFFF), bool d = false, bool b = false);

		RGBA col;
		bool draw : 1;
		bool base : 1;
	};

	class enumeration;

public:
	typedef enumeration ENUMERATION;

	PenBase(const PenBase &) throw(HGL::ImageOutput::ImageOutputException);
	PenBase(const char *const penDesc) throw(ImageOutputException);
	_EXPORT PenBase(const RGBA &baseColor = 0xFFFFFFFF,
					std::size_t dim = 1) throw(ImageOutputException);

	_EXPORT virtual ~PenBase();

	PenBase &operator=(const PenBase &) throw(ImageOutputException);
	PenBase &operator=(const char *) throw(ImageOutputException);

	int getDimension() const _PURE;
	const ENUMERATION getEnumeration() const _PURE;

	inline bool isStandardPen() const {
		return m_stdPen;
	}

	PenBase getNilPen() const;
	const RGBA &getBaseColor() const _CONST;
	void setBaseColor(const RGBA &bc);
	void setPenDescription(const char *const penDesc) throw(ImageOutputException);

private:

	_NOUNUSED PenData *init(const char *const penDesc) throw(ImageOutputException);
	_NOUNUSED PenData *init(const RGBA &baseColor, std::size_t dim) throw(ImageOutputException);

private:
	std::size_t m_center;
	bool m_stdPen;
	int m_dim;
	PenData *m_data;
	RGBA m_baseColor;
};

class ICanvas::PenBase::enumeration {
	friend const enumeration PenBase::getEnumeration() const;

	inline enumeration(std::size_t dim, const PenData *pd) : cur(0), end(dim *dim),
		data(pd) {}

	mutable std::size_t cur;
	std::size_t end;
	const PenData *data;

public:
	typedef PenData PENDATA;

	inline bool hasNext() const {
		return cur < end;
	}

	inline const PENDATA &next() const {
		return data[cur++];
	}
};

}

}

namespace std {

extern template
iterator_traits<HGL::ImageOutput::ICanvas::COLORSPACE::const_iterator>::difference_type
distance<HGL::ImageOutput::ICanvas::COLORSPACE::const_iterator>
(HGL::ImageOutput::ICanvas::COLORSPACE::const_iterator,
 HGL::ImageOutput::ICanvas::COLORSPACE::const_iterator);

extern template iterator_traits<HGL::ImageOutput::ICanvas::RGBA *>::difference_type
distance<HGL::ImageOutput::ICanvas::RGBA *>(HGL::ImageOutput::ICanvas::RGBA *,
		HGL::ImageOutput::ICanvas::RGBA *);

}

#endif // HGL_IMAGEOUTPUT_ICANVAS_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
