/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

/**
 * @file icontrolsource.h
 * @brief Macros and interfaces to the control source subsystem
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/

#ifndef HGL_ICONTROLSOURCE_H
#define HGL_ICONTROLSOURCE_H

#include <vector>
#include <string>

#include <stdint.h>

#include "ipluginpoptoptions.h"
#include "plugininfobase.h"

/**
 * @defgroup csplugins Control source plugins
 * @brief Classes, methods, etc. related to the development of control source plugins
 *
 * @cond PLUGIN_API
 * See @ref cspluginsdetail for more information.
 * @endcond
 */

#ifndef INTERPRETER
/**
 * @ingroup csplugins
 * @def REGISTER_CSPLUGIN
 * @hideinitializer
 * @brief Registers the control source plugin with HGL
 *
 * Put this macro into the C++-implementation file of your plugin.
 * Example of usage:
 * @code
 * REGISTER_CSPLUGIN(CGIControlSource,
 *                   200,
 *                   L"CGI Control Source",
 *                   L"Heiko Schäfer",
 *                   L"© 2013 by Heiko Schäfer <heiko@hgl.rangun.de>",
 *                   L"Takes input from the Common Gateway Interface")
 * @endcode
 *
 * @relates IControlSource
 *
 * @param Typename     the class name of your plugin
 * @param Priority     the priority of this plugin
 * @param Name         the name of this plugin
 * @param Author       the author of this plugin
 * @param Copyright    a copyright message
 * @param Description  a @b short description of the plugin
 *
 **/
#define REGISTER_CSPLUGIN(Typename, Priority, Name, Author, Copyright, Description) \
	HGL::ControlSource::PLUGININFO _pluginInfo(0L, Name, Author, Copyright, Description, 0L, Priority, new Typename());
#endif

namespace HGL {

class IType;

namespace Type {
class IParam;
}

namespace ControlSource {

/**
 * @ingroup csplugins
 * @interface IControlSource
 * @brief Interface to the control source subsystem
 *
 * @cond PLUGIN_API
 * See @ref cspluginsdetail for more information.
 * @endcond
 *
 * @see REGISTER_CSPLUGIN
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/
class __attribute__((__packed__)) IControlSource : public HGL::Common::IPluginPoptOptions {
	IControlSource(const IControlSource &);
	IControlSource &operator=(const IControlSource &);

public:
	virtual ~IControlSource() = 0;

	/**
	 * @brief A vector of Parameter
	 **/
	typedef std::vector<const HGL::Type::IParam *> PARAMVECTOR;

	/**
	 * @brief Gets the PARAMVECTOR for an given procedure
	 *
	 * @param procedure the procedure to get the parameters for
	 * @return the PARAMVECTOR for the given procedure
	 **/
	virtual PARAMVECTOR getParams(const std::string &procedure) const = 0;

protected:
	inline IControlSource() : IPluginPoptOptions() {}
};

inline IControlSource::~IControlSource() {}

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
typedef struct __attribute__((__packed__)) _pluginInfo : public Common::_pluginInfoBase {
#ifndef WIN32
	inline _pluginInfo(void *phandle, const wchar_t *pname, const wchar_t *pauthor,
#else
	inline _pluginInfo(HINSTANCE phandle, const wchar_t *pname, const wchar_t *pauthor,
#endif
					   const wchar_t *pcopyright, const wchar_t *pdescription, IType *proot,
					   uint8_t ppriority, const HGL::ControlSource::IControlSource *pcontrolSource)
		: _pluginInfoBase(phandle, pname, pauthor, pcopyright, pdescription),
		controlSource(pcontrolSource), root(proot), priority(ppriority) {}
const HGL::ControlSource::IControlSource *controlSource;
IType *root;
uint8_t priority;
} PLUGININFO;
#pragma GCC diagnostic pop

#ifndef INTERPRETER
extern "C" HGL::ControlSource::PLUGININFO _pluginInfo;
#endif

}

}

extern template class std::vector<const HGL::Type::IParam *>;

#endif // HGL_ICONTROLSOURCE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
