/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

/**
 * @file ioutput.h
 * @brief Macros and interfaces to the image output subsystem
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/

#ifndef HGL_IMAGEOUTPUT_IOUTPUT_H
#define HGL_IMAGEOUTPUT_IOUTPUT_H

#include "imageoutputexception.h"
#include "ipluginpoptoptions.h"
#include "plugininfobase.h"

#undef _pluginInfo
#undef PLUGININFO

/**
 * @defgroup ioplugins Image output plugins
 * @brief Classes, methods, etc. related to the development of image output plugins
 *
 * @cond PLUGIN_API
 * See @ref iopluginsdetail for more information.
 * @endcond
 */

#ifndef INTERPRETER
/**
 * @ingroup ioplugins
 * @def REGISTER_IOPLUGIN
 * @hideinitializer
 * @brief Registers the image output plugin with HGL
 *
 * Put this macro into the C++-implementation file of your plugin.
 * Example of usage:
 * @code
 * REGISTER_IOPLUGIN(PNGOutput,
 *                   "PNG",
 *                   L"PNG Image Output",
 *                   L"Heiko Schäfer",
 *                   L"© 2013 by Heiko Schäfer <heiko@hgl.rangun.de>",
 *                   L"writes PNG images")
 * @endcode
 *
 * @relates IOutput
 *
 * @param Typename     the class name of your plugin
 * @param ImageType    the image type this plugin produces
 * @param Name         the name of this plugin
 * @param Author       the author of this plugin
 * @param Copyright    a copyright message
 * @param Description  a @b short description of the plugin
 *
 **/
#define REGISTER_IOPLUGIN(Typename, ImageType, Name, Author, Copyright, Description) \
	HGL::ImageOutput::PLUGININFO _pluginInfo(0L, Name, Author, Copyright, Description, ImageType, \
			new Typename());
#endif

namespace HGL {

namespace Interpreter {
class Interpreter;
}

namespace ImageOutput {

class ICanvas;

/**
 * @ingroup ioplugins
 * @interface IOutput
 * @brief Interface for writing images into a file.
 *
 * @cond PLUGIN_API
 * See @ref iopluginsdetail for more information.
 * @endcond
 *
 * @see REGISTER_IOPLUGIN
 *
 * @deprecated New plugins have to use the interface IPreRenderOutput. This interface exists only
 * for compatibility to pre 0.4.70 plugins.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/
class _EXPORT IOutput : public HGL::Common::IPluginPoptOptions {
	IOutput(const IOutput &);
	IOutput &operator=(const IOutput &);

public:
	virtual ~IOutput() = 0;

	/**
	 * @name Information functions
	 * @{
	 */

	/**
	 * @brief Get the effective output filename of the rendered picture
	 *
	 * It should return an empty @c std::string if the standard output is used
	 *
	 * @return the output filename of the rendered picture
	 */
	virtual std::string getOutputFilename() const = 0;

	//@}

	/**
	 * @name Bitmap render methods
	 * @{
	 */

	/**
	 * @brief Writes an ICanvas bitmap into an image
	 *
	 * @param canvas the ICanvas bitmap to write
	 * @param filename filename to write the image to, if the filename is the empty string it
	 * 					should write to the standard output
	 * @exception ImageOutputException if an unrecoverable error occured while writing
	 *
	 * @return @c true if HGL should not do any more actions, @c false otherwise
	 *
	 * @see Writer
	 **/
	virtual bool writeImage(const ICanvas &canvas,
							const std::string &filename =
								std::string()) const throw(ImageOutputException) = 0;

	//@}

protected:
	IOutput();
};

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
typedef struct __attribute__((__packed__)) _pluginInfo : public Common::_pluginInfoBase {
#ifndef WIN32
	_pluginInfo(void *phandle, const wchar_t *pname, const wchar_t *pauthor,
#else
	_pluginInfo(HINSTANCE phandle, const wchar_t *pname, const wchar_t *pauthor,
#endif
				const wchar_t *pcopyright, const wchar_t *pdescription,
				const char *const pimageType, const IOutput *pImageOutput) :
		_pluginInfoBase(phandle, pname, pauthor, pcopyright, pdescription),
		imageType(const_cast<char *>(pimageType)), imageOutput(pImageOutput) {}
char			*imageType;
const IOutput	*imageOutput;
} PLUGININFO;
#pragma GCC diagnostic pop

#ifndef INTERPRETER
extern "C" HGL::ImageOutput::PLUGININFO _pluginInfo;
#endif

}

}

#endif // HGL_IMAGEOUTPUT_IOUTPUT_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
