/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_IPARAM_H
#define HGL_TYPE_IPARAM_H

#include "itype.h"

namespace HGL {

namespace Type {

/**
 * @defgroup params Parameter types
 *
 * @see AssignmentOperator
 * @see DebugProcedure
 * @see ParamSection
 * @see ProcCall
 * @see ProcInitStatement
 * @see RenderCallback
 * @see StdLibCall
 *
 * @ingroup types
 * @ingroup params
 * @interface IParam
 *
 * @brief This interface marks a type as parameter type
 *
 * Parameter types are used as parameters to procedures
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED IParam : public virtual HGL::IType {
	DISALLOW_COPY_AND_ASSIGN(IParam)
	friend class ParamBase;
public:
	using IType::operator-;
	using IType::operator*;
	using IType::operator=;

	virtual ~IParam() = 0;

	/**
	 * @brief The types of operations available for expressions
	 **/
	typedef enum _INTERNAL {
		NOP, ///< no operation
		ADD, ///< addition
		SUB, ///< subtraction
		MUL, ///< multiplication
		DIV, ///< division
		POW, ///< power
		SIN, ///< sinus
		COS, ///< cosinus
		NEG, ///< negation
		DAO  ///< direct assignemnt operation
	} OP;

	/**
	 * @brief Gets the string representation of a parameter type
	 *
	 * @return the string representation of a parameter type
	 */
	virtual const std::string toString() const = 0;

	/**
	 * @brief Checks equality of parameter types
	 *
	 * @param  o the other parameter type to check
	 * @return @c true, if they are equal, @c false otherwise
	 */
	virtual bool operator==(const IParam &o) const = 0;

protected:
	_INTERNAL inline IParam() {}

	_NOUNUSED virtual const IType *findReference(const IType *) const = 0;

	template<class Operator, class Fallback>
	_INTERNAL IType *operate(const IType &, Operator op,
							 Fallback fallback) throw(Exception::InvalidExpressionException);
};

inline IParam::~IParam() {}

}

}

#endif // HGL_TYPE_IPARAM_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
