/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_IPARSER_H
#define HGL_COMMON_IPARSER_H

#include "parserexception.h"
#include "ioexception.h"

namespace HGL {

namespace Type {
class IProject;
class IProjectFactory;
}

namespace Common {

class URI;

/**
 * @ingroup util
 * @interface IParser
 *
 * @brief Interface to a parsing subsystem
 *
 * This interface provides methods needed for HGL::Common::createProject
 * Concrete subclasses should implement a parser, which returns the ProjectInterface as root of
 * the type tree and use Factory to create the types.
 *
 * @see HGL::Type::IProject
 * @see HGL::Type::IProjectFactory
 * @see HGL::Common::createProject
 *
 * @tparam ProjectInterface the type of root type
 * @tparam Factory type of the factory to create types
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 *
 **/
template<class ProjectInterface = HGL::Type::IProject, class Factory = HGL::Type::IProjectFactory>
class _INTERNAL IParser {
	DISALLOW_COPY_AND_ASSIGN(IParser)
public:
	virtual ~IParser() {}

	/**
	 * @brief Parses an input source and returns a type tree
	 *
	 * @throws HGL::Exception::ParserException if there has been a parsing error
	 * @throws HGL::Exception::IOException if there has been an I/O error
	 *
	 * @param infile the input file
	 * @param factory factory to use to create types
	 * @param lib root node of the standard library
	 * @return the root node of the type tree
	 */
	_NOUNUSED virtual ProjectInterface *parse(const Common::URI &infile, Factory *const factory,
			ProjectInterface *const lib = 0) const
	throw(HGL::Exception::ParserException, HGL::Exception::IOException) = 0;

	/**
	 * @overload
	 **/
	_NOUNUSED virtual ProjectInterface *parse(Factory *const factory,
			ProjectInterface *const lib) const
	throw(HGL::Exception::ParserException, HGL::Exception::IOException) = 0;

	/**
	 * @brief Gets the opening mode of the input file
	 *
	 * @c "rb" for binary input or @c "r" for text input
	 *
	 * @return the opening mode of the input file
	 */
	virtual const char *getMode() const = 0;

	/**
	 * @brief Cleanup function after parsing or if an exception is thrown
	 */
	virtual void cleanup() const = 0;

protected:
	inline IParser() {}
};

}

}

#endif /* HGL_COMMON_IPARSER_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
