/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @file iprerenderoutput.h
 * @brief Macros and interfaces to the image output subsystem
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.70
 **/

#ifndef HGL_IMAGEOUTPUT_IPRERENDEROUTPUT_H
#define HGL_IMAGEOUTPUT_IPRERENDEROUTPUT_H

#include "ioutput.h"
#include "icanvas.h"

namespace HGL {

namespace ImageOutput {

struct CurvePrimitive;
struct EllipsePrimitive;
struct LinePrimitive;
struct PointPrimitive;
struct PolyPrimitive;
struct TextPrimitive;

/**
* @ingroup ioplugins
* @interface IPreRenderOutput
* @brief Interface for writing images into a file.
*
* @note If you need to keep the primitives you got from the update methods, make a copy of them,
* since it is not guaranteed they exist over all the lifetime of the plugin.
*
* @cond PLUGIN_API
* See @ref iopluginsdetail for more information.
* @endcond
*
* @see REGISTER_IOPLUGIN
*
* @author Heiko Schäfer <heiko@hgl.rangun.de>
* @since 0.4.70
* @copyright BSD License
**/
class _EXPORT IPreRenderOutput : public IOutput {
	IPreRenderOutput(const IPreRenderOutput &);
	IPreRenderOutput &operator=(const IPreRenderOutput &);
public:

	/**
	* @enum RENDEREVENT
	* @brief Indicates the state of rendering
	* @since 0.4.70
	*/
	typedef enum _INTERNAL _PACKED {
		/// overall rendering has started
		START,
		/// overall rendering has ended
		END
	} RENDEREVENT;

	/**
	 * @name Pre-render notifications
	 * @{
	 */

	/**
	 * @brief Notification about an general render event
	 *
	 * @param evt the event
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.4.70
	 **/
	virtual void update(const RENDEREVENT evt) throw(ImageOutputException) = 0;

	/**
	 * @brief Notification about transformation information available
	 *
	 * @param ti the transformation information
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.5.2
	 **/
	virtual void update(const ICanvas::TRANSFORMATIONINFO *const ti) throw(ImageOutputException) = 0;

	/**
	 * @brief Notification about a point to render
	 *
	 * @param primitive the point primitive
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.4.70
	 **/
	virtual void update(const PointPrimitive *const primitive) throw(ImageOutputException) = 0;

	/**
	 * @brief Notification about a line to render
	 *
	 * @param primitive the line primitive
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.4.70
	 **/
	virtual void update(const LinePrimitive *const primitive) throw(ImageOutputException) = 0;

	/**
	* @brief Notification about a curve to render
	*
	* @param primitive the curve primitive
	*
	* @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	* @since 0.5.0
	**/
	virtual void update(const CurvePrimitive *const primitive) throw(ImageOutputException) = 0;

	/**
	 * @brief Notification about a poly to render
	 *
	 * @param primitive the poly primitive
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.4.70
	 **/
	virtual void update(const PolyPrimitive *const primitive) throw(ImageOutputException) = 0;

	/**
	 * @brief Notification about an ellipse to render
	 *
	 * @param primitive the ellipse primitive
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.4.70
	 **/
	virtual void update(const EllipsePrimitive *const primitive) throw(ImageOutputException) = 0;

	/**
	 * @brief Notification about a text to render
	 *
	 * @param primitive the text primitive
	 *
	 * @throws HGL::ImageOutput::ImageOutputException in case of an unrecoverable error
	 * @since 0.4.70
	 **/
	virtual void update(const TextPrimitive *const primitive) throw(ImageOutputException) = 0;

	//@}

protected:
	IPreRenderOutput();
	~IPreRenderOutput();
};

}

}

#endif // HGL_IMAGEOUTPUT_IPRERENDEROUTPUT_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
