/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_IPROJECT_H
#define HGL_TYPE_IPROJECT_H

#include "itype.h"

namespace HGL {

namespace Common {
class IApplication;
class URI;
}

namespace Type {

class MainSegment;
class ObjectsSegment;
class ProceduresSegment;

/**
 * @ingroup types
 * @interface IProject
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED IProject : public virtual IType {
	DISALLOW_COPY_AND_ASSIGN(IProject)
public:

	using IType::operator=;

	/**
	 * @brief Adds the main segment
	 *
	 * @param ms the main segment
	 * @return the attached main segment
	 **/
	_INTERNAL _NOUNUSED virtual MainSegment *addMainSegment(MainSegment *ms) = 0;

	_INTERNAL virtual MainSegment *getMainSegment() const = 0;

	/**
	 * @brief Adds the objects segment
	 *
	 * @param os the objects segment
	 * @return the attached objects segment
	 **/
	_INTERNAL _NOUNUSED virtual ObjectsSegment *addObjectsSegment(ObjectsSegment *os) = 0;

	_INTERNAL virtual ObjectsSegment *getObjectsSegment() const = 0;

	/**
	 * @brief Adds the procedures segment
	 *
	 * @param ps the procedures segment
	 * @return the attached procedures segment
	 **/
	_INTERNAL _NOUNUSED virtual ProceduresSegment *addProceduresSegment(ProceduresSegment *ps) = 0;

	_INTERNAL virtual ProceduresSegment *getProceduresSegment() const = 0;

	typedef struct _PACKED {
		bool reserved1		: 1;
		bool unzipped		: 1;
		bool compactColors	: 1;
		bool gpgSigned		: 1;
		bool rleEncoding	: 1;
		bool cspTarget		: 1;
		bool hasLogOp		: 1;
		bool twoByteStrings	: 1;
		bool omitDesc		: 1;
		bool reserved10		: 1;
		bool reserved11		: 1;
		bool reserved12		: 1;
		bool reserved13		: 1;
		bool reserved14		: 1;
		bool reserved15		: 1;
		bool reserved16		: 1;
	} COMPILERFLAGS;

	_INTERNAL COMPILERFLAGS getCompilerFlags() const {
		return m_compilerFlags;
	}

	_INTERNAL void setCompilerFlags(const COMPILERFLAGS &cf) {
		m_compilerFlags = cf;
	}

	typedef struct version {
		inline bool operator<(const version &v) const {
			return maj < v.maj ||
				   (maj == v.maj && min < v.min) ||
				   (maj == v.maj && min == v.min && rel < v.rel);
		}

		inline bool operator==(const version &v) const {
			return v.maj == maj && v.min == min && v.rel == rel;
		}

		inline bool operator!=(const version &v) const {
			return !(v == *this);
		}

		uint16_t maj;
		uint16_t min;
		uint16_t rel;
	} VSTRUCT;

protected:
	_INTERNAL IProject() : IType(), m_compilerFlags() {
		m_compilerFlags.reserved1		= false;
		m_compilerFlags.unzipped		= true;
		m_compilerFlags.compactColors	= true;
		m_compilerFlags.gpgSigned		= false;
		m_compilerFlags.rleEncoding		= true;
		m_compilerFlags.cspTarget		= true;
		m_compilerFlags.hasLogOp		= false;
		m_compilerFlags.twoByteStrings	= true;
		m_compilerFlags.omitDesc		= true;
		m_compilerFlags.reserved10		= false;
		m_compilerFlags.reserved11		= false;
		m_compilerFlags.reserved12		= false;
		m_compilerFlags.reserved13		= false;
		m_compilerFlags.reserved14		= false;
		m_compilerFlags.reserved15		= false;
		m_compilerFlags.reserved16		= false;
	}

public:
	static VSTRUCT HGL_CURRENT_VERSION;

private:
	COMPILERFLAGS m_compilerFlags;
};

}

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
