/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_IPROJECTFACTORY_H
#define HGL_TYPE_IPROJECTFACTORY_H

#include "projectfactoryexception.h"
#include "referenceexception.h"
#include "conditionbase.h"

namespace HGL {

class IType;

namespace Type {

class AssignmentOperator;
class ProcInitStatement;
class ProceduresSegment;
class IReferenceHolder;
class RenderProcedure;
class ExpressionParam;
class RenderCallback;
class DebugProcedure;
class ObjectsSegment;
class ReferenceParam;
class ConditionBlock;
class ParamSection;
class VirtualGroup;
class StringParam;
class MainSegment;
class NamedParam;
class StdLibCall;
class FloatParam;
class Signature;
class Procedure;
class Condition;
class ProcCall;
class IProject;
class Vector2D;
class Include;
class Ellipse;
class IfElse;
class IPoint;
class IParam;
class Curve;
class Point;
class Poly;
class Fill;
class Line;
class Text;
class Pen;

/**
 * @ingroup util
 * @ingroup types
 * @interface IProjectFactory
 *
 * @brief Interface to a factory creating types.
 *
 * @see HGL::Common::IParser
 * @see HGL::Common::createProject
 * @see HGL::Type::DefaultProjectFactory
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _INTERNAL IProjectFactory {
	DISALLOW_COPY_AND_ASSIGN(IProjectFactory)
public:
	virtual ~IProjectFactory() {}

	/**
	 * @brief Creates an @c IProject
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual IProject *createProject(const std::string &id,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c MainSegment
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual MainSegment *createMainSegment(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Include
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Include *createInclude(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c ObjectsSegment
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ObjectsSegment *createObjectsSegment(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c ProceduresSegment
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ProceduresSegment *createProceduresSegment(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Signature
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Signature *createSignature(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c VirtualGroup
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual VirtualGroup *createVirtualGroup(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c ConditionBlock
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ConditionBlock *createConditionBlock(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Procedure
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Procedure *createProcedure(const std::string &id,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c StdLibCall
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual StdLibCall *createStdLibCall(IType *lib = 0L,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c ProcCall
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ProcCall *createProcCall(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c NamedParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual NamedParam *createNamedParam(const std::string &typeName,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c ReferenceParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ReferenceParam *createProcParam(const std::string &ref,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;
	/**
	 * @brief Creates a @c FloatParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual FloatParam *createProcParam(float f,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;
	/**
	 * @brief Creates an @c IParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual IParam *createProcParam(const IParam *,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c ProcInitStatement
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ProcInitStatement *createProcInitStatement(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c Ellipse
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Ellipse *createEllipse(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Point
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Point *createPoint(bool markerPoint, int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c IPoint
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual IPoint *createPoint(const std::string &id, const IPoint &p, bool markerPoint,
										  int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Line
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Line *createLine(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Line
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Line *createLine(IPoint *p1, IPoint *p2, int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Curve
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Curve *createCurve(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Poly
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Poly *createPoly(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Fill
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Fill *createFill(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Text
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Text *createText(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Vector2D
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Vector2D *createVector2D(float x, float y,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c DebugProcedure
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual DebugProcedure *createDebugProcedure(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c RenderProcedure
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual RenderProcedure *createRenderProcedure(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c RenderCallback
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual RenderCallback *createRenderCallback(const std::string &ref,
			IType *stdlib, int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c StringParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual StringParam *createStringParam(const std::string &txt,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c ParamSection
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ParamSection *createParamSection(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c AssignmentOperator
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual AssignmentOperator *createAssignmentOperator(const std::string &id,
			IType *type = 0L, int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c ExpressionParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ExpressionParam *createExpressionParam(const std::string &,
			const IProjectFactory *, int line = -1) const throw(Exception::ReferenceException,
					Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c ExpressionParam
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual ExpressionParam *createExpressionParam(const ExpressionParam *,
			const IProjectFactory *, int line = -1) const throw(Exception::ReferenceException,
					Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c NullType
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual IType *createNullType(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates a @c Condition
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Condition *createCondition(const HGL::Type::ConditionBase::OP &op,
			int line = -1) const throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c IfElse
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual IfElse *createIfElse(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

	/**
	 * @brief Creates an @c Pen
	 * @param line an optional line number or @c -1 (default) for none
	 */
	_NOUNUSED virtual Pen *createPen(int line = -1) const
	throw(Exception::ProjectFactoryException) = 0;

protected:
	inline IProjectFactory() {}
};

}

}

#endif /* HGL_TYPE_IPROJECTFACTORY_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
