/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_TYPE_IREFERENCEHOLDER_H
#define HGL_TYPE_IREFERENCEHOLDER_H

#include "itype.h"

namespace HGL {

namespace Type {

/**
 * @ingroup types
 * @interface IReferenceHolder
 * @brief Interface to IType which can refer to other types
 *
 * A type can hold a reference to another IType and should behave like that type.
 * A reference itself is only a string to an identifier.
 *
 * @see IType::getId
 * @see IType::setId
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT IReferenceHolder : public virtual IType {
	IReferenceHolder &operator=(const IReferenceHolder &);
protected:
	template<typename T> struct PropertyTraits;
public:
	using IType::operator=;

	virtual ~IReferenceHolder() = 0;

	/**
	 * @name Reference access and manipulation
	 * @{
	 **/

	/**
	 * @brief Gets the reference
	 *
	 * @return pointer to the reference, @c NULL if no reference is set
	 **/
	_NOUNUSED virtual const std::string *getReference() const = 0;

	/**
	 * @brief Sets the reference
	 *
	 * @param ref the reference
	 * @param fromStdLib <code>true</code> if the reference points into the stdlib
	 *
	 * @throw ReferenceException if setting the reference failed
	 *
	 **/
	virtual void setReference(const std::string &ref, bool fromStdLib = false)
	throw(Exception::ReferenceException) = 0;

	/**
	 * @brief Sets the reference
	 *
	 * Scoped references are useful for types which are created at runtime.
	 *
	 * @param ref the reference
	 * @param scope the scope in which the reference should be found
	 **/
	virtual void setReference(const std::string &ref, IType *scope) = 0;

	/**
	 * @brief Removes the reference
	 *
	 * @pre The underlying type has to be valid (isValid() == @c true), i.e. if the reference is
	 * removed the resulting type must be valid, otherwise an ReferenceRemoveException is thrown
	 *
	 * @see isValid
	 *
	 * @throws HGL::Exception::ReferenceRemoveException if the underlying type is not valid
	 *
	 **/
	virtual void removeReference() throw(Exception::ReferenceException) = 0;

	/**
	 * @brief Searches the tree for the associated reference
	 *
	 * @note To check if a reference was actually found use IType::isNull
	 *
	 * @see IType::isNull
	 * @return an IType to a reference or the special NullType if not found
	 **/
	_NOUNUSED virtual const IType *findReference(bool recache = false) const = 0;

	///@}

	/**
	 * @brief Calculates the hash value
	 *
	 * @return uint16_t the hash value
	 */
	virtual uint16_t getHash() const = 0;

	template<typename T>
	bool isPropertyEqual(const typename PropertyTraits<T>::type *) const throw();

protected:
	inline IReferenceHolder() {}
};

inline IReferenceHolder::~IReferenceHolder() {}

template<typename T> struct IReferenceHolder::PropertyTraits {
	typedef T type;
	bool operator()(const type *, const type *) const throw() {
		return false;
	}
};

template<typename T>
bool IReferenceHolder::isPropertyEqual(const typename PropertyTraits<T>::type *t) const throw() {
	if(t) {
		return ((t->getReference() && getReference()) &&
				(*(t->getReference()) == *getReference())) ? true :
			   PropertyTraits<T>()(t, dynamic_cast<const typename PropertyTraits<T>::type *>(this));
	}

	return false;
}

}

}

#endif /* HGL_TYPE_IREFERENCEHOLDER_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
