/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

/**
 * @file itype.h
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/

/**
 * @defgroup types Types
 * @brief The <code>types</code> supported by the <em>HGL Suite</em>
 *
 * The graph below shows the relationships between types.
 * @dotfile typerels.dot
 * At left you see the types and to the right the types allowed to attach.
 *
 * Type relationship violations are typically indicated by @c HGL::Exception::AttachException or
 * by returning @c false at @c HGL::IType::isValid .
 **/

#ifndef HGL_ITYPE_H
#define HGL_ITYPE_H

/**
 * @def ACCEPT
 * @hideinitializer
 * @brief Marks the type to be visitable by a subclass of @c HGL::Type::ITypeVisitor
 * @param TYPE name of the type
 **/
#define ACCEPT(TYPE) virtual void accept(HGL::Type::ITypeVisitor &)

/**
 * @def ACCEPT_IMPL
 * @hideinitializer
 * @brief Adds implementation code to be visitable by a subclass of @c HGL::Type::ITypeVisitor
 * @param TYPE name of the type
 **/
#define ACCEPT_IMPL(TYPE) void TYPE::accept(HGL::Type::ITypeVisitor &v) { \
		v.visit(this); \
	}

#include <list>

#include <stdint.h>

#include "clonenotsupportedexception.h"
#include "invalidexpressionexception.h"
#include "referenceexception.h"
#include "attachexception.h"
#include "nr_traverser.h"
#include "basevector2d.h"

/**
 * @brief Global namespace of the <em>HGL Suite</em>
 **/
namespace HGL {

/**
 * @brief This namespace contains all type classes
 **/
namespace Type {
class ITypeVisitor;
class StringParam;
class Vector2D;
class SerialID;
}

/**
 * @ingroup types
 * @interface IType
 * @brief Base interface of types
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT IType {
	IType(const IType &);
	template<class T> friend void std::swap(T &, T &);

protected:
	class Ptr;
	struct _boundingbox;

public:
	/**
	 * @name Serialization and type identification
	 * @{
	 **/

	/**
	 * @brief Gets the serialization ID of the object
	 *
	 * @return the serialization ID of the object
	 **/
	virtual const Type::SerialID &getSerialID() const throw() = 0;

	///@}

	/**
	 * @name Identification and description of types
	 * @{
	 **/

	/**
	 * @brief Gets the identifier
	 *
	 * @return the identifier
	 **/
	virtual const std::string &getId() const = 0;

	/**
	 * @brief Sets the identifier
	 *
	 * @param id the identifier
	 **/
	virtual void setId(const std::string &id) = 0;

	/**
	 * @brief Gets the description of the type
	 *
	 * @return the description of the type
	 **/
	virtual const std::string &getDescription() const = 0;

	/**
	 * @brief Sets the description of the type
	 *
	 * @param desc the description of the type
	 *
	 **/
	virtual void setDescription(const std::string &desc) = 0;

	///@}

	/**
	 * @name Accessing attached types
	 * @{
	 **/

	/**
	 * @brief Get the root of a type
	 *
	 * @param type the type to get the root of
	 * @return the the root of a type
	 */
	virtual const IType *getRoot(const IType &type) const = 0;

	virtual void setRoot(IType *) = 0;

	/**
	 * @brief Returns the parent @c IType
	 *
	 * @return the parent @c IType or <code>NULL</code> if there is no parent
	 **/
	_NOUNUSED virtual IType *getParent() const = 0;

	/**
	 * @brief Set parent of the type
	 *
	 * @param parent the parent of the type
	 */
	virtual void setParent(IType *parent) = 0;

	/**
	 * @brief Resolves a type by its ID
	 *
	 * @param id the id to resolve
	 * @return the resolved type or <code>0L</code> if no type could get resolved by that id
	 */
	_NOUNUSED virtual IType *resolve(const std::string &id) const = 0;

	/**
	 * @brief List of attached @c IType instances
	 **/
	typedef std::deque<Ptr> TYPELIST;

	/**
	 * @brief Returns all attached @c IType instances
	 *
	 * @return all attached @c IType instances
	 **/
	_NOUNUSED virtual const TYPELIST &getAttachedTypes() const throw() = 0;

	/**
	 * @brief Attach child objects to the @c IType at end of list
	 *
	 * @param type the @c IType to attach as child
	 * @param check @c true (default), if validity checks should be performed
	 * @throws Exception::AttachException if a disallowed type gets attached.
	 * See @ref types for more.
	 *
	 * @return the attached type
	 **/
	virtual IType *attachType(IType *type, bool check = true) throw(Exception::AttachException) = 0;

	/**
	 * @brief Detaches a child from the IType
	 *
	 * @param type the IType to detach
	 *
	 * @since 0.5.21
	 **/
	virtual void detachType(IType *type) _NONNULL(2) = 0;

	///@}

	/**
	 * @name Managing control source parameters
	 * @{
	 **/

	virtual void setControlSourceParameter(uint32_t par)
	throw(Exception::InvalidExpressionException) = 0;
	virtual uint32_t getControlSourceParameter() const = 0;

	///@}

	/**
	 * @name Type conversion
	 * @{
	 **/

	/**
	 * @brief Converts the type into a <code>float</code>
	 *
	 * @return the type as <code>float</code>
	 */
	virtual operator float() const throw(Exception::ReferenceException,
										 Exception::InvalidExpressionException) = 0;

	/**
	 * @brief Converts the type into a <code>HGL::Type::Vector2D</code>
	 *
	 * @return the type as <code>HGL::Type::Vector2D</code> reference
	 */
	virtual operator Type::Vector2D &() const throw(Exception::InvalidExpressionException) = 0;

	/**
	 * @brief Converts the type into a <code>HGL::Type::StringParam</code>
	 *
	 * @return the type as <code>HGL::Type::StringParam</code>
	 */
	virtual operator Type::StringParam &() const throw(Exception::InvalidExpressionException) = 0;

	///@}

	/**
	 * @name Aritmetic operators
	 * @{
	 **/

	virtual IType &operator=(const IType &) throw(Exception::InvalidExpressionException) = 0;
	virtual IType *operator+(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) = 0;
	virtual IType *operator-(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) = 0;
	virtual IType *operator*(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) = 0;
	virtual IType *operator*(const float *) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) = 0;
	virtual IType *operator/(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) = 0;
	virtual IType *operator-() throw(Exception::InvalidExpressionException) = 0;

	virtual IType *power(const IType &) throw(Exception::InvalidExpressionException) = 0;
	virtual IType *sinus() throw(Exception::InvalidExpressionException) = 0;
	virtual IType *cosinus() throw(Exception::InvalidExpressionException) = 0;

	///@}

	virtual ~IType() = 0;

	/**
	 * @brief Creates a clone of the type
	 *
	 * @return a smart pointer to the cloned type
	 */
	virtual _NOUNUSED IType::Ptr clone() const throw(Exception::CloneNotSupportedException) = 0;

	ACCEPT() = 0;

	/**
	 * @name Iterator over this type's attached types
	 * @{
	 **/

	class iterator;

	/**
	 * @brief Gets the iterator over this type's attached types
	 *
	 * @return the iterator over this type's attached types
	 */
	virtual operator iterator() = 0;

	virtual void invalidateIterator(bool parent = false) const = 0;

	/**
	 * @brief Gets the iterator to the begin over this type's attached types
	 *
	 * @return the iterator to the begin over this type's attached types
	 */
	virtual iterator begin() = 0;

	/**
	 * @brief Gets the iterator to the end over this type's attached types
	 *
	 * @return the iterator to the end over this type's attached types
	 */
	virtual iterator end() = 0;

	///@}

	/**
	 * @name Type checks
	 * @{
	 **/

	/**
	 * @brief Checks if the type is null type
	 * @see IReferenceHolder::findReference
	 * @return <code>true</code> if the type is null type, <code>false</code> otherwise
	 */
	virtual bool isNull() const throw() = 0;

	/**
	 * @brief Checks if the type is valid
	 * @return  <code>true</code> if the type is valid, <code>false</code> otherwise
	 **/
	virtual bool isValid() const throw() = 0;

	///@}

	/**
	 * @brief The bounding box over the type
	 * @author Heiko Schäfer <heiko@hgl.rangun.de>
	 */
	typedef _boundingbox BOUNDINGBOX;

	/**
	 * @brief Gets the bounding box of the type
	 *
	 * @return the bounding box of the type
	 */
	virtual BOUNDINGBOX getBoundingBox() const = 0;

	virtual const IType &operator->() const = 0;

	friend _EXPORT bool operator==(const IType::Ptr &, const Type::SerialID &);
	friend _EXPORT bool operator!=(const IType::Ptr &, const Type::SerialID &);

public:
	static const uint32_t NOCSP _EXPORT;

protected:
	inline IType() {}
};

inline IType::~IType() {}

/**
 * @brief Smart pointer to @c IType instances
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT IType::Ptr {
public:
	inline Ptr(const Ptr &p) throw() : m_ptr(p.m_ptr) {}
	inline Ptr(IType *const p) throw() : m_ptr(p) {}
	inline ~Ptr() throw() {}

	inline Ptr &operator=(Ptr o) {
		swap(o);
		return *this;
	}

	inline operator bool() const {
		return m_ptr;
	}

	inline friend bool operator==(const Ptr &l, IType *r) {
		return l.m_ptr == r;
	}

	inline friend bool operator!=(const Ptr &l, IType *r) {
		return !(l == r);
	}

	inline friend bool operator<(const Ptr &l, IType *r) {
		return l.m_ptr < r;
	}

	inline const IType *operator->() const {
		return &m_ptr->operator->();
	}

	inline const IType &operator*() const {
		return *static_cast<IType *>(m_ptr);
	}

	inline const IType *getRawPointer() const {
		return m_ptr;
	}

	template<class T>
	_HOT inline operator T *() const throw() {
		return dynamic_cast<T *>(m_ptr);
	}

	friend inline void swap(Ptr &l, Ptr &r) {
		l.swap(r);
	}

	inline void swap(Ptr &o) throw() {
		using std::swap;

		swap(m_ptr, o.m_ptr);
	}

private:
	IType *m_ptr;
};

struct _INTERNAL IType::_boundingbox {
	/**
	 * @brief Contructor
	 *
	 * Creates an undefined (not yet calculated) bounding box
	 */
	_EXPORT _boundingbox();

	/**
	 * @brief Constructor
	 * @param tl the top left corner
	 * @param br the bottom right corner
	 */
	inline _boundingbox(const HGL::Type::BaseVector2D &tl, const HGL::Type::BaseVector2D &br) :
		topLeft(tl), bottomRight(br) {}

	/**
	 * @brief Constructor
	 *
	 * Convenience version to directly give the positions as <code>float</code>
	 *
	 * @param tl_x the top left x-coordinate
	 * @param tl_y the top left y-coordinate
	 * @param br_x the bottom right x-coordinate
	 * @param br_y the bottom right y-coordinate
	 */
	inline _boundingbox(float tl_x, float tl_y, float br_x, float br_y) :
		topLeft(HGL::Type::BaseVector2D(tl_x, tl_y)),
		bottomRight(HGL::Type::BaseVector2D(br_x, br_y)) {}

	friend inline void swap(_boundingbox &l, _boundingbox &r) {
		using std::swap;

		swap(l.topLeft, r.topLeft);
		swap(l.bottomRight, r.bottomRight);
	}

	_EXPORT _boundingbox &operator=(_boundingbox o) {
		swap(*this, o);
		return *this;
	}

	_EXPORT bool operator<(const _boundingbox &o) const _PURE;
	_EXPORT bool operator==(const _boundingbox &o) const _PURE;

	/**
	 * @brief Checks if the bounding box is defined (i.e. yet calculated)
	 *
	 * @return <code>true</code> if the bounding box is defined, <code>false</code> otherwise
	 */
	_EXPORT bool isDefined() const _PURE;

	/**
	 * @brief Calculates the aspect ratio of the type
	 *
	 * @return the aspect ratio
	 */
	_EXPORT HGL::Type::BaseVector2D getAspectRatio() const;

	/**
	 * @brief Calculates the centroid of the bounding box
	 *
	 * @return the centroid of the bounding box
	 */
	inline HGL::Type::BaseVector2D getCentroid() const {
		return
			HGL::Type::BaseVector2D(topLeft.getX() + (bottomRight.getX() - topLeft.getX()) / 2.0f,
									topLeft.getY() + (bottomRight.getY() - topLeft.getY()) / 2.0f);
	}

	/**
	 * @brief Calculates the width of the bounding box
	 *
	 * @return the width of the bounding box
	 */
	inline float getWidth() const {
		return fabsf(bottomRight.getX() - topLeft.getX());
	}

	/**
	 * @brief Calculates the height of the bounding box
	 *
	 * @return the height of the bounding box
	 */
	inline float getHeight() const {
		return fabsf(bottomRight.getY() - topLeft.getY());
	}

	HGL::Type::BaseVector2D topLeft;
	HGL::Type::BaseVector2D bottomRight;

};

template<> inline _HOT IType::Ptr::operator HGL::IType *() const throw() {
	return m_ptr;
}

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
/**
 * @brief Iterator over an IType tree
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class IType::iterator : public std::iterator<std::forward_iterator_tag, IType *> {
	friend iterator IType::begin();
	friend iterator IType::end();
	friend IType::operator iterator();

	friend void Commons::traverse<TYPELIST::const_iterator>(
		IType *,
		TYPELIST::const_iterator( &)(const IType *),
		TYPELIST::const_iterator( &)(const IType *),
		IType::iterator &, void ( &)(const IType *), void ( &)());

public:
	explicit iterator(IType *const);
	iterator(const iterator &);
	virtual ~iterator();

	iterator &operator=(iterator);

	bool operator!=(const iterator &) const _PURE;
	bool operator==(const iterator &) const _PURE;

	const iterator &operator--() const;
	const iterator &operator++();
	const iterator operator++(int);

	IType *operator*() const _PURE;
	IType *operator->() const _PURE;

	iterator begin() const;
	iterator end() const;

	friend inline void swap(iterator &l, iterator &r) {
		using std::swap;

		l.swap(r);
	}

	void swap(iterator &o) throw();

private:
	Commons::COND operator()(IType *);

private:
	typedef std::vector<Ptr> TYPEVECTOR;

	TYPEVECTOR *m_typesVector;
	TYPEVECTOR::size_type m_pos;
};
#pragma GCC diagnostic pop

_EXPORT bool operator==(const IType *, const Type::SerialID &);
_EXPORT bool operator!=(const IType *, const Type::SerialID &);

_EXPORT bool operator==(const IType &, const Type::SerialID &);
_EXPORT bool operator!=(const IType &, const Type::SerialID &);

}

extern template class std::deque<uint8_t, std::allocator<uint8_t> >;
extern template class std::deque<HGL::IType *, std::allocator<HGL::IType *> >;
extern template class std::list<HGL::IType::Ptr, std::allocator<HGL::IType::Ptr> >;
extern template class std::vector<HGL::IType *, std::allocator<HGL::IType *> >;
extern template class std::vector<HGL::IType::Ptr, std::allocator<HGL::IType::Ptr> >;
extern template class std::deque<HGL::IType::Ptr, std::allocator<HGL::IType::Ptr> >;
extern template struct std::iterator<std::forward_iterator_tag, HGL::IType *>;

extern template void Commons::traverse<HGL::IType::TYPELIST::const_iterator>(
	HGL::IType *,
	HGL::IType::TYPELIST::const_iterator( &)(const HGL::IType *),
	HGL::IType::TYPELIST::const_iterator( &)(const HGL::IType *),
	HGL::IType::iterator &, void ( &)(const HGL::IType *), void ( &)());

namespace std {

template<> void swap<HGL::IType::Ptr>(HGL::IType::Ptr &lhs, HGL::IType::Ptr &rhs);
template<> void swap<HGL::IType::iterator>(HGL::IType::iterator &lhs, HGL::IType::iterator &rhs);

}

#endif // HGL_ITYPE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
