/*
 * nr_traverser.h - this template contains a non-recursive function to traverse n-trees
 *
 * $Revision: 2751 $ $Author: heiko $
 *
 * (c) 2012-2014 Heiko Schäfer <heiko@hgl.rangun.de>
 *
 * Distribution of this file is only allowed if the author is mentioned.
 * Modifications of this algorithm have to be reported to the author.
 *
 */

#ifndef COMMONS_NR_TRAVERSER_H
#define COMMONS_NR_TRAVERSER_H

#include <vector>
#include <stack>

namespace Commons {

/**
 * @brief A condition on how to proceed before traversing child nodes
 *
 **/
typedef enum _PACKED {
	/// Continue traversing the children of the current node
	CONTINUE,
	/// Discard all children of the current node
	DISCARD,
	/// Break, i.e. stop traversing at this point
	BREAK
} COND;

/**
 *
 * @tparam Iterator type of the iterator to use for traversal
 * @tparam Type type of object iterated over
 * @tparam Begin functor to get the begin iterator to the children of the current node
 * @tparam End functor to get the end iterator to the children of the current node
 * @tparam Top functor called before traversing child nodes
 * @tparam Middle functor called after completely traversing all children of current node
 * @tparam Bottom functor called after traversing a node
 *
 **/
template < typename Iterator, typename Type, typename Begin, typename End, typename Top,
		 typename Middle, typename Bottom >
/**
 * @brief Traverses a n-tree (i.e. a tree with 0-n children on every node), performing actions on
 * every node
 *
 * @note This algorithm works non-recusive.
 *
 * The middle functor takes a node as argument:
 * @code
 * void middle(const HGL::IType *);
 * @endcode
 *
 * The top functor additionally returns Algorithm::COND:
 * @code
 * Algorithm::COND top(const HGL::IType *);
 * @endcode
 *
 * The function should be called with the type of iterator as template parameter:
 * @code
 * Algorithm::traverse<MYTYPE::iterator>(node, begin, end, top, middle, bottom);
 * @endcode
 *
 * @param root Node to start traversing
 * @param getBeginIterator functor to get the begin iterator to the children of the current node
 * @param getEndIterator functor to get the end iterator to the children of the current node
 * @param top functor called before traversing child nodes
 * @param middle functor called after completely traversing all children of current node
 * @param bottom functor called after traversing a node
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
void traverse(Type root, Begin &getBeginIterator, End &getEndIterator, Top &top, Middle &middle,
			  Bottom &bottom) {

	std::stack<std::pair<Type, Iterator>, std::vector<std::pair<Type, Iterator> > > stack;
	stack.push(std::make_pair(root, getBeginIterator(root)));

	while(!stack.empty()) {

		const COND &c(top(stack.top().first));

		if(c == CONTINUE) {
			while(!stack.empty()) {

				typename std::stack < std::pair<Type, Iterator>,
						 std::vector < std::pair < Type,
						 Iterator > > >::reference stackTop(stack.top());

				if(stackTop.second != getEndIterator(stackTop.first)) {
					const typename Iterator::reference ref(*stackTop.second++);
					stack.push(std::make_pair(ref, getBeginIterator(ref)));
					break;
				} else {
					middle(stackTop.first);
					stack.pop();
				}
			}

			bottom();
		}

		if(c == BREAK) break;
	}
}

}

#endif /* COMMONS_NR_TRAVERSER_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
