/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_PARAMETERFACTORY_H
#define HGL_PARAMETERFACTORY_H

#include "icontrolsource.h"
#include "singleton.h"

namespace HGL {

namespace Type {
class Vector2D;
class FloatParam;
class StringParam;
class ReferenceParam;
}

namespace ControlSource {

/**
 * @ingroup csplugins
 * @brief Factory for creating parameters to hand over to the interpreter
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/
class ParameterFactory : public HGL::Common::Singleton<ParameterFactory> {
	ParameterFactory(const ParameterFactory &);
	ParameterFactory &operator=(const ParameterFactory &);

	friend class HGL::Common::Singleton<ParameterFactory>;

public:
	/**
	 * @brief Clears the ControlSource::IControlSource::PARAMVECTOR
	 *
	 * @param pv a ControlSource::IControlSource::PARAMVECTOR
	 **/
#ifdef WIN32
	_EXPORT
#endif
	void clearParamVector(const ControlSource::IControlSource::PARAMVECTOR &pv) const;

	/**
	 * @name Methods to create parameters
	 * @{
	 */

	/**
	 * @brief Creates an @c StringParam instance
	 *
	 * @param s the string, the StringParam should represent
	 * @return an StringParam instance
	 **/
#ifdef WIN32
	_EXPORT
#endif
	HGL::Type::StringParam *createStringParameter(const std::string &s) const;

	/**
	 * @brief Creates an @c FloatParam instance
	 *
	 * @param f the float value, the FloatParam should represent
	 * @return an FloatParam instance
	 **/
#ifdef WIN32
	_EXPORT
#endif
	HGL::Type::FloatParam *createNumericParameter(float f) const;

	/**
	 * @brief Creates an @c Vector2D instance
	 *
	 * @param x the x-value, the Vector2DParam should represent
	 * @param y the y-vale, the Vector2DParam should represent
	 * @return an Vector2D instance
	 **/
#ifdef WIN32
	_EXPORT
#endif
	HGL::Type::Vector2D *createNumericParameter(float x, float y) const;

	/**
	 * @brief Creates an @c FloatParam instance
	 *
	 * @param i the integer value, the FloatParam should represent
	 * @return an FloatParam instance
	 **/
#ifdef WIN32
	_EXPORT
#endif
	HGL::Type::FloatParam *createNumericParameter(int i) const;

	/**
	 * @brief Creates an ReferenceParam instance
	 *
	 * @param project TODO
	 * @param ref the reference, the ReferenceParam should represent
	 * @return an ReferenceParam instance
	 **/
#ifdef WIN32
	_EXPORT
#endif
	HGL::Type::ReferenceParam *createReferenceParameter(IType *project,
			const std::string &ref) const _NONNULL(2);

	//@}

private:
	inline ParameterFactory() {}
};

}

}

#endif // HGL_PARAMETERFACTORY_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
