/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_PARSER_H
#define HGL_COMMON_PARSER_H

#include <ctime>

#include "ioexception.h"
#include "parserexception.h"
#include "poptexitexception.h"

struct poptOption;

namespace HGL {

extern Common::URI *handlePopt(const char *const mode, const std::string &progdesc,
							   const std::string &version, const std::string &buildnode,
							   const std::string &buildhost, const std::string &target,
							   std::time_t builddate, int argc, const char **argv,
							   struct ::poptOption *const progOpts, const char *const suffix,
							   char **outputFilename) throw(HGL::Exception::PoptExitException,
									   HGL::Exception::IOException) _NONNULL(1, 12);

namespace Type {
class IProject;
class IProjectFactory;
}

namespace Common {

template<class, class> class IParser;
class URI;

/**
 * @ingroup util
 * @brief Creates a project
 *
 * Parses and creates the internal tree of a <em>HGL Suite</em> project
 *
 * @tparam ProjectInterface type of the project interface
 * @tparam Factory type of the project factory
 * @param progdesc name of the component, i.e. <em>interpreter</em>, <em>compiler</em>,
 * <em>svg2hgl</em>
 * @param version version of the component
 * @param buildnode name of the node where it was built
 * @param buildhost the platform on which the <em>HGL Suite</em> was compiled
 * @param target the platform for which the <em>HGL Suite</em> was compiled
 * @param builddate date of the build as Unix time
 * @param argc the comandline argument count
 * @param argv the command line arguments
 * @param parser the parser to use for parsing the input
 * @param fac the factory to use for building the types
 * @param progOpts tbw
 * @param outputFilename the name of the output file
 * @param suffix suffix to append to @c outputFilename if omitted or @c NULL
 * @param lib the stdlib or <code>0L</code> if the project is the stdlib itself
 *
 * @return the parsed project tree
 */
template<class ProjectInterface, class Factory>
_EXPORT ProjectInterface *createProject(const std::string &progdesc, const std::string &version,
										const std::string &buildnode, const std::string &buildhost,
										const std::string &target, std::time_t builddate,
										int argc, const char **argv,
										const HGL::Common::IParser < ProjectInterface,
										Factory > *const parser, Factory *const fac,
										struct ::poptOption *const progOpts,
										char **outputFilename, const char *const suffix,
										ProjectInterface *const lib = 0L)
throw(HGL::Exception::ParserException, HGL::Exception::IOException,
	  HGL::Exception::PoptExitException) _NONNULL(9, 10, 12);

template<class ProjectInterface, class Factory>
ProjectInterface *createProject(const std::string &progdesc, const std::string &version,
								const std::string &buildnode, const std::string &buildhost,
								const std::string &target, std::time_t builddate, int argc,
								const char **argv, const HGL::Common::IParser < ProjectInterface,
								Factory > *const parser, Factory *const fac,
								struct ::poptOption *const progOpts, char **outputFilename,
								const char *const suffix, ProjectInterface *const lib)
throw(HGL::Exception::ParserException, HGL::Exception::IOException,
	  HGL::Exception::PoptExitException) {

	return createProject(parser, fac, HGL::handlePopt(parser->getMode(), progdesc, version,
						 buildnode, buildhost, target, builddate, argc, argv, progOpts,
						 suffix, outputFilename), lib);
}

/**
 * @ingroup util
 * @overload
 **/
template<class ProjectInterface, class Factory>
_EXPORT ProjectInterface *createProject(const HGL::Common::IParser < ProjectInterface,
										Factory > *const parser, Factory *const fac,
										HGL::Common::URI *const infile,
										ProjectInterface *const lib = 0L)
throw(HGL::Exception::ParserException, HGL::Exception::IOException) _NONNULL(1, 2);

template<class ProjectInterface, class Factory>
ProjectInterface *createProject(const HGL::Common::IParser < ProjectInterface,
								Factory > *const parser, Factory *const fac,
								HGL::Common::URI *const infile,
								ProjectInterface *const lib)
throw(HGL::Exception::ParserException, HGL::Exception::IOException) {

	ProjectInterface *p = 0L; // does syntactic parse

	if(infile) {
		try {
			p = parser->parse(*infile, fac, lib);
		} catch(HGL::Exception::IOException &e) {
			parser->cleanup();
			e.setURI(*infile);
			throw;
		}
	} else {
		p = parser->parse(fac, lib);
	}

	return p;
}

}

}

#endif /* HGL_COMMON_PARSER_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
