/*
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef HGL_COMMON_POPTTABLEBUILDER_H
#define HGL_COMMON_POPTTABLEBUILDER_H

#include <string>

#include "linkercontrol.h"

struct poptOption;

namespace HGL {

namespace Common {

class PoptTableBuilderImpl;

/**
 * @ingroup util
 * @brief Builds a POPT table structure
 *
 * This class builds a POPT table structure for dynamic inclusion of POPT command line arguments.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.5.15
 */
class _EXPORT PoptTableBuilder {
	DISALLOW_COPY_AND_ASSIGN(PoptTableBuilder)
public:
	PoptTableBuilder(struct ::poptOption *const origOpts);
	~PoptTableBuilder();

	/**
	 * @brief Builds a reference to a POPT option table to include
	 *
	 * @pre A table built by calls to @c addSubTable and @c addEntry
	 *
	 * @return a reference to a POPT option table to include
	 *
	 * @since 0.5.15
	 */
	struct ::poptOption &getPoptTable() const;

	struct ::poptOption *getOrigOptionsTable() const _PURE;

	/**
	 * @brief Adds an entry to the last added subtable
	 *
	 * @param name for the POPT option table
	 * @param opt POPT option structure array
	 *
	 * @since 0.5.15
	 */
	void addEntry(const std::string &name, struct ::poptOption *opt) const;

	/**
	 * @brief Adds a subtable (group) containing further POPT tables
	 *
	 * @param desc a description for the group
	 *
	 * @since 0.5.15
	 */
	void addSubtable(const std::string &desc) const;

	/**
	 * @brief Checks if a given POPT option is @c POPT_ARG_INCLUDE_TABLE and its @c arg
	 * is @c NULL
	 *
	 * @param opt a POPT option
	 *
	 * @return @c true if the given POPT option is @c POPT_ARG_INCLUDE_TABLE and its @c arg
	 * is @c NULL, @c false otherwise
	 *
	 * @since 0.5.15
	 */
	static bool isTableEntry(const struct ::poptOption &opt) _PURE;

private:
	PoptTableBuilderImpl *const _pimpl;
};

}

}

#endif /* HGL_COMMON_POPTTABLEBUILDER_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
