/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/**
 * @file primitives.h
 * @brief The pre-render primitive structs
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.70
 **/

#ifndef HGL_IMAGEOUTPUT_PRIMITIVES_H
#define HGL_IMAGEOUTPUT_PRIMITIVES_H

#include "imageoutputexception.h"
#include "icanvas.h"

#ifndef _PACKED
#define _PACKED __attribute__((__packed__))
#endif

#define STDPEN HGL::ImageOutput::ICanvas::PEN()

namespace HGL {

namespace Interpreter {

namespace Render {
class IFiller;
class Canvas;
}

}

namespace ImageOutput {

struct PolyPrimitive;

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
struct _INTERNAL Primitive {

	inline explicit Primitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) :
		canvas(0L), filler(0L), id(), pen(pen) {}
	virtual ~Primitive();

	_NOUNUSED virtual PolyPrimitive *
	asPolyPrimitive(const ICanvas::TRANSFORMATIONINFO &) const {
		return 0L;
	}

	virtual void render(const ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException) = 0;

	/// the canvas to which to render the primitive
	HGL::Interpreter::Render::Canvas *canvas;

	/// the IFiller
	const HGL::Interpreter::Render::IFiller *filler;

	/// the id of the primitive
	std::string id;

	mutable HGL::ImageOutput::ICanvas::PEN pen;
};
#pragma GCC diagnostic pop

/**
 * @ingroup ioplugins
 * @brief A point primitive
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.4.70
 * @copyright BSD License
 **/
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
struct _INTERNAL PointPrimitive : public Primitive {
	inline PointPrimitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) : Primitive(pen),
		mutated(0L), color(), x(), y(), mval(0x0), mask(false), markerPoint(true),
		unscaled(false) {}

	virtual ~PointPrimitive();

	_NOUNUSED virtual PolyPrimitive *asPolyPrimitive(const ICanvas::TRANSFORMATIONINFO &ti) const;

	virtual void render(const ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException);

protected:
	void renderMutated(const ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException);

public:
	PolyPrimitive *mutated;

/// the color
	ICanvas::RGBA color;

/// the x-coordinate
	float x;

/// the y-coordinate
	float y;

/// the mask value
	uint8_t mval;

/// is the point masked?
	bool mask : 3;

/// is the point a marker point?
	bool markerPoint : 3;

/// is the point unscaled?
	bool unscaled : 1;

protected:
	_NOUNUSED PolyPrimitive *toPolyPrimitive(float x1, float y1, float x2, float y2,
			const ICanvas::RGBA &col1, const ICanvas::RGBA &col2,
			const ICanvas::TRANSFORMATIONINFO &ti) const;
};
#pragma GCC diagnostic pop

/**
 * @ingroup ioplugins
 * @brief A line primitive
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.4.70
 * @copyright BSD License
 **/
struct _INTERNAL LinePrimitive : public PointPrimitive {
	inline LinePrimitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) : PointPrimitive(pen),
		p1(pen), p2(pen), scaled(false) {}
	~LinePrimitive();

	_NOUNUSED virtual PolyPrimitive *asPolyPrimitive(const ICanvas::TRANSFORMATIONINFO &ti) const;

	virtual void render(const ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException);

/// the starting point of the line
	PointPrimitive p1;

/// the ending point of the line
	PointPrimitive p2;

/// need to apply scaling if false
	bool scaled;
};

/**
 * @ingroup ioplugins
 * @brief A poly primitive
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.4.70
 * @copyright BSD License
 **/
struct _INTERNAL PolyPrimitive : public LinePrimitive {
	inline PolyPrimitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) : LinePrimitive(pen),
		scaleX(1.0f), edgeTable(), lineList(), col1(), col2(), offX(0.0f) {}
	virtual ~PolyPrimitive();
	virtual void render(const ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException);

	float scaleX;

	EDGETABLE edgeTable;
	LINELIST lineList;

	/// start color of the color gradient
	ICanvas::RGBA col1;

	/// end color of the color gradient
	ICanvas::RGBA col2;

	float offX;
};

/**
 * @ingroup ioplugins
 * @brief A curve primitive
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.5.0
 * @copyright BSD License
 **/
struct _INTERNAL CurvePrimitive : public PolyPrimitive {
	inline CurvePrimitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) : PolyPrimitive(pen),
		lines(), distance(0.0f), cp1(pen), cp2(pen), numCp(0) {}
	~CurvePrimitive();
	virtual void render(const ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException);

	/// the calculated polygon lines of the curve
	std::vector<LinePrimitive *> lines;

	/// the distance between the curve's start- and endpoint
	float distance;

	/// the first control point of the curve
	PointPrimitive cp1;

	/// the second control point of the curve
	PointPrimitive cp2;

	/// number of control points
	size_t numCp;
};

/**
 * @ingroup ioplugins
 * @brief An ellipse primitive
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.4.70
 * @copyright BSD License
 **/
struct _INTERNAL EllipsePrimitive : public PointPrimitive {
	inline EllipsePrimitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) :
		PointPrimitive(pen), p(pen), erx(), ery(), fcol1(), fcol2(), fill(false), centric(false) {}
	virtual void render(const HGL::ImageOutput::ICanvas::TRANSFORMATIONINFO &ti) const
	throw(HGL::ImageOutput::ImageOutputException);

/// the center point of the ellipse
	PointPrimitive p;

/// the x-radius
	float erx;

/// the y-radius
	float ery;

/// start color of the color gradient
	ICanvas::RGBA fcol1;

/// end color of the color gradient
	ICanvas::RGBA fcol2;

/// is the ellipse filled?
	bool fill : 4;

/// centric fill of ellipse?
	bool centric : 4;
};

/**
 * @ingroup ioplugins
 * @brief A text primitive
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.4.70
 * @copyright BSD License
 **/
struct _INTERNAL TextPrimitive : public PointPrimitive {
	inline TextPrimitive(const HGL::ImageOutput::ICanvas::PEN &pen = STDPEN) : PointPrimitive(pen),
		fontSize(), font(), text() {}
	virtual void render(const ICanvas::TRANSFORMATIONINFO &) const
	throw(HGL::ImageOutput::ImageOutputException) {}

	/// the font size
	int fontSize;

	/// the font to render with
	std::string font;

	/// the text to render
	std::string text;
};

}

}

extern template class std::set<HGL::ImageOutput::BUCKET>;
extern template class std::vector<HGL::ImageOutput::BUCKET>;
extern template class std::map < int_fast16_t, HGL::ImageOutput::BUCKETSET,
	   std::greater<int_fast16_t> >;

#endif // HGL_IMAGEOUTPUT_PRIMITIVES_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
