/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_QFORMAT_H
#define HGL_QFORMAT_H

#include <algorithm>

#include <stdint.h>

#include "linkercontrol.h"

#define Q24_CONFIG_FIT BYTES_7
#define Q24_CONFIG_BYTES 7

extern const long double fromQ24;

namespace HGL {

namespace Common {

/**
 * @ingroup util
 * @brief Class to represent a float as 24bit integer in the Q number format
 *
 * See https://en.wikipedia.org/wiki/Q_(number_format) for more information
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class Q24 {
public:
	_EXPORT explicit Q24(int64_t q) throw();

	_EXPORT Q24(int64_t ri, bool) throw();

	_EXPORT Q24(float f) throw();

	_EXPORT Q24() throw();

	inline float toFloat() const throw() {
		return static_cast<float>((static_cast<long double>(m_data * fromQ24)));
	}

	inline int64_t asInteger() const throw() {
		return m_data;
	}

	inline bool operator==(const Q24 &o) const throw() {
		return m_data == o.m_data;
	}

	inline bool operator!=(const Q24 &o) const throw() {
		return !(m_data == o.m_data);
	}

	inline bool operator<(const Q24 &o) const throw() {
		return m_data < o.m_data;
	}

	inline bool operator>(const Q24 &o) const throw() {
		return m_data > o.m_data;
	}

	inline Q24 &operator+=(const Q24 &o) throw() {
		m_data += o.m_data;
		return *this;
	}

	inline Q24 operator+(const Q24 &o) const throw() {
		return Q24(*this) += o;
	}

	inline Q24 &operator-=(const Q24 &o) throw() {
		m_data -= o.m_data;
		return *this;
	}

	inline Q24 operator-(const Q24 &o) const throw() {
		return Q24(*this) -= o;
	}

	inline Q24 &operator*=(const Q24 &r) throw() {
		m_data = (m_data * r.m_data) + static_cast<int64_t>(K);
		m_data >>= 24;
		return *this;
	}

	inline Q24 operator*(const Q24 &o) const throw() {
		return Q24(*this) *= o;
	}

	_EXPORT Q24 &operator/=(const Q24 &r) throw();

	inline Q24 operator/(const Q24 &o) const throw() {
		return Q24(*this) /= o;
	}

	inline Q24 operator-() const throw() {
		return Q24(-m_data);
	}

	friend inline Q24 operator+(const Q24 &l, float r) throw() {
		return Q24(l) += Q24(r);
	}

	friend inline Q24 operator-(const Q24 &l, float r) throw() {
		return Q24(l) -= Q24(r);
	}

	friend inline Q24 operator*(float l, const Q24 &r) throw() {
		return Q24(l) *= r;
	}

	friend inline Q24 operator/(const Q24 &l, float r) throw() {
		return Q24(l) /= Q24(r);
	}

	friend inline Q24 operator/(float l, const Q24 &r) throw() {
		return Q24(l) /= r;
	}

	friend inline Q24 operator-(const Q24 &r) throw() {
		return r.operator - ();
	}

	friend inline void swap(Q24 &l, Q24 &r) throw() {
		using std::swap;

		swap(l.m_data, r.m_data);
	}

	inline Q24 &operator=(Q24 o) throw() {
		swap(*this, o);
		return *this;
	}

	_EXPORT Q24 &inflateRLE(uint64_t ri);
	_EXPORT bool deflateRLE(uint64_t *ri) const _NONNULL(2);

private:
	static const uint64_t BYTES_1 = uint64_t(0xFFULL);
	static const uint64_t BYTES_2 = uint64_t(0xFFFFULL);
	static const uint64_t BYTES_3 = uint64_t(0xFFFFFFULL);
	static const uint64_t BYTES_4 = uint64_t(0xFFFFFFFFULL);
	static const uint64_t BYTES_5 = uint64_t(0xFFFFFFFFFFULL);
	static const uint64_t BYTES_6 = uint64_t(0xFFFFFFFFFFFFULL);
	static const uint64_t BYTES_7 = uint64_t(0xFFFFFFFFFFFFFFULL);
	static const uint64_t BYTES_8 = uint64_t(-1LL);

	static int64_t inflateRLE_internal(uint64_t ri) throw() _CONST;
	static bool deflateRLE_internal(uint64_t *ri, int64_t val, uint64_t fit) _NONNULL(1);

private:
	int64_t m_data;

	static const long double K;
};

}

}

#endif // HGL_QFORMAT_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
