/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_RAIIFACTORY_H
#define HGL_RAIIFACTORY_H

#include <set>

#include "util.h"

namespace HGL {

class IType;

namespace Common {

template<class T, bool arrayDel, class Container, std::size_t capacity>
class _INTERNAL RAIIAdaptor {
	DISALLOW_COPY_AND_ASSIGN(RAIIAdaptor)

	typedef Container CONTAINER;

public:
	inline explicit RAIIAdaptor() : m_container() {}

	inline CONTAINER &getContainer() {
		return m_container;
	}

	inline void reserve(std::size_t s) {
		if(m_container.size() < s) {
			m_container.reserve(s);
		}
	}

	inline void insert(T *p) _NONNULL(2) {

		if(m_container.size() == m_container.capacity()) {
			m_container.reserve(std::min(m_container.capacity() + capacity,
										 m_container.max_size()));
		}

		m_container.push_back(p);
	}

	bool erase(const T *p) _NONNULL(2) {

		if(m_container.back() == p) {
			m_container.pop_back();
			return true;
		} else {
			size_t size = m_container.size();
			m_container.erase(std::remove(m_container.begin(), m_container.end(), p),
							  m_container.end());
			return (m_container.size() < size);
		}

		return false;
	}

private:
	CONTAINER m_container;
};

template<class T, bool arrayDel, std::size_t capacity>
class _INTERNAL RAIIAdaptor<T, arrayDel, std::set<const T *>, capacity> {
	DISALLOW_COPY_AND_ASSIGN(RAIIAdaptor)

	typedef std::set<const T *> CONTAINER;

public:
	inline explicit RAIIAdaptor() : m_container() {}

	inline CONTAINER &getContainer() {
		return m_container;
	}

	static inline void reserve(std::size_t) {}

	inline void insert(T *p) _NONNULL(2) {
		m_container.insert(p);
	}

	bool erase(const T *p) _NONNULL(2) {

		const typename CONTAINER::const_iterator &i(m_container.find(p));

		if(i != m_container.end()) {
			m_container.erase(i);
		}

		return false;
	}

private:
	CONTAINER m_container;
};

/**
 * @ingroup util
 * @brief Template to implement a RAII factory
 *
 * @note There exists an specialisation for @c std::set to improve performance if
 * there are many calls to @c RAIIFactory::release
 *
 * @tparam T the type to keep track of
 * @tparam Container type of container to store references
 * @tparam arrayDel flag to indicate that @c T is an array
 * @tparam capacity the capacity to hold before minimizing the memory usage
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
template < class T, class Container = std::vector<const T *>, bool arrayDel = false,
		 std::size_t capacity = 50 >
class _EXPORT RAIIFactory {
	DISALLOW_COPY_AND_ASSIGN(RAIIFactory)

	typedef RAIIAdaptor<T, arrayDel, Container, capacity> ADAPTOR;

public:
	inline RAIIFactory() : m_pushCnt(0u), m_adaptor() {}

	void reserve(std::size_t s) const;

	T *keep(T *p) const;

	T *release(T *p) const _NONNULL(2);

	template<typename IIter>
	void release(IIter first, IIter last) {

		if(m_adaptor.getContainer().empty()) return;

		std::for_each(first, last,
					  std::bind1st(std::mem_fun < void, RAIIFactory < T, Container, arrayDel,
								   capacity > , const T * > (&RAIIFactory < T, Container, arrayDel,
										   capacity >::release), this));
	}

	~RAIIFactory();

private:
	mutable size_t m_pushCnt;
	mutable ADAPTOR m_adaptor;
};

template<class T, class Container, bool arrayDel, std::size_t capacity>
void RAIIFactory<T, Container, arrayDel, capacity>::reserve(std::size_t s) const {
	m_adaptor.reserve(s);
}

template<> _CONST inline void
RAIIFactory<HGL::IType, std::set<const HGL::IType *>, false, 50u>::reserve(std::size_t s) const {
	m_adaptor.reserve(s);
}

template<class T, class Container, bool arrayDel, std::size_t capacity>
T *RAIIFactory<T, Container, arrayDel, capacity>::keep(T *p) const {

	if(p) {
		m_adaptor.insert(p);

		if(m_pushCnt >= capacity) {
			m_pushCnt = 0u;
			Util::shrink_to_fit(m_adaptor.getContainer());
		}
	}

	return p;
}

template<class T, class Container, bool arrayDel, std::size_t capacity>
T *RAIIFactory<T, Container, arrayDel, capacity>::release(T *p) const {

	if(m_adaptor.getContainer().empty()) return p;

	if(m_adaptor.erase(p)) {
		Util::shrink_to_fit(m_adaptor.getContainer());
	}

	return p;
}

template<class T, class Container, bool arrayDel, std::size_t capacity>
RAIIFactory<T, Container, arrayDel, capacity>::~RAIIFactory() {

	const Container &c(m_adaptor.getContainer());
	typename Container::const_iterator i(c.begin());
	const typename Container::const_iterator &e(c.end());

	if(!arrayDel) {
		for(; i != e; ++i) delete *i;
	} else {
		for(; i != e; ++i) delete [] *i;
	}
}

}

}

#endif // HGL_RAIIFACTORY_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
