/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_REGEXINITIALIZER_H
#define HGL_REGEXINITIALIZER_H

#include <regex.h>

#include "linkercontrol.h"

namespace HGL {

/**
 * @brief This namespace contains common utility classes
 **/
namespace Common {

/**
 * @ingroup util
 *
 * @brief Initializes an extended regular expression
 *
 * Regular expressions must be compiled before use and freed after use.
 * Therefore it is adviseable to do this in static initialization and destruction.
 * This class will initialize, resp. compile a regular expression in its constructor and freeing
 * it in its destructor.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _INTERNAL RegexInitializer {
	DISALLOW_COPY_AND_ASSIGN(RegexInitializer)
public:
	/**
	 * @brief %Constructor
	 *
	 * Initializes, resp. compiles the regular expression
	 *
	 * @param pattern the pattern to compile
	 * @param nosub %true if the REG_NOSUB flag should be set
	 */
	_EXPORT explicit RegexInitializer(const char *const pattern, bool nosub = false) _NONNULL(2);

	/**
	 * @brief %Destructor
	 *
	 * Frees the regular expression
	 */
	_EXPORT virtual ~RegexInitializer();

	/**
	 * @brief Gets the compiled regular expression
	 * @return the compiled regular expression
	 **/
	_EXPORT inline operator const regex_t *() const {
		return &m_regex;
	}

private:
	regex_t m_regex; ///< the compiled regular expression
};

}

}

#endif // HGL_REGEXINITIALIZER_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
