/***
 * Copyright (c) 2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

/**
 * @file serialid.h
 * @brief Serialization identifier
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @since 0.5.26
 * @copyright BSD License
 **/

#ifndef HGL_TYPE_SERIALID_H
#define HGL_TYPE_SERIALID_H

#include <string>

#include <stdint.h>

#include "linkercontrol.h"

namespace HGL {

namespace Type {

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Weffc++"
/**
 * @ingroup types
 * @brief Serialization identifier class
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.5.26
 **/
class SerialID {
public:
	/**
	 * @brief Enum of the serializeable objects
	 *
	 * This types are used to denote the type of the object
	 *
	 * @note Do not alter the order of this enum since it will break compatibility!
	 * If you need to reorder values here, change the global min version in CompilerBase,
	 * if compiled signatures changed or new signatures are introduced, mark it in the appropriate
	 * CompilerType subclass.
	 **/
	typedef enum {
		SERIALID_END = -99,
		SERIALID_ANY = -98,
		SERIALID_NOTSERIALIZEABLE = -1, ///< Denotes a non-serializeable type
		SERIALID_ASSIGNMENTOPERATOR, ///< Serial type id of <code>HGL::Type::AssignmentOperator</code>
		SERIALID_DEBUGPROCEDURE, ///< Serial type id of <code>HGL::Type::DebugProcedure</code>
		SERIALID_ELLIPSE, ///< Serial type id of <code>HGL::Type::Ellipse</code>
		SERIALID_EXPRESSIONPARAM, ///< Serial type id of <code>HGL::Type::ExpressionParam</code>
		SERIALID_FILL, ///< Serial type id of <code>HGL::Type::Fill</code>
		SERIALID_FLOATPARAM, ///< Serial type id of <code>HGL::Type::FloatParam</code>
		SERIALID_LINE, ///< Serial type id of <code>HGL::Type::Line</code>
		SERIALID_MAINSEGMENT, ///< Serial type id of <code>HGL::Type::MainSegment</code>
		SERIALID_NAMEDPARAM, ///< Serial type id of <code>HGL::Type::NamedParam</code>
		SERIALID_OBJECTSSEGMENT, ///< Serial type id of <code>HGL::Type::ObjectsSegment</code>
		SERIALID_PARAMSECTION, ///< Serial type id of <code>HGL::Type::ParamSection</code>
		SERIALID_POINT, ///< Serial type id of <code>HGL::Type::Point</code>
		SERIALID_POLY, ///< Serial type id of <code>HGL::Type::Poly</code>
		SERIALID_PROCCALL, ///< Serial type id of <code>HGL::Type::ProcCall</code>
		SERIALID_PROCEDURE, ///< Serial type id of <code>HGL::Type::Procedure</code>
		SERIALID_PROCEDURESSEGMENT, ///< Serial type id of <code>HGL::Type::ProceduresSegment</code>
		SERIALID_PROCINITSTATEMENT, ///< Serial type id of <code>HGL::Type::ProcInitStatement</code>
		SERIALID_PROJECT, ///< Serial type id of <code>HGL::Type::Project</code>
		SERIALID_REFERENCEPARAM, ///< Serial type id of <code>HGL::Type::ReferenceParam</code>
		SERIALID_RENDERPROCEDURE, ///< Serial type id of <code>HGL::Type::RenderProcedure</code>
		SERIALID_STDLIBCALL, ///< Serial type id of <code>HGL::Type::StdLibCall</code>
		SERIALID_STRINGPARAM, ///< Serial type id of <code>HGL::Type::StringParam</code>
		SERIALID_TEXT, ///< Serial type id of <code>HGL::Type::Text</code>
		SERIALID_VECTOR2D, ///< Serial type id of <code>HGL::Type::Vector2D</code>
		SERIALID_RENDERCALLBACK, ///< Serial type id of <code>HGL::Type::RenderCallback</code>
		SERIALID_SRENDERCALLBACK, ///< Serial type id of <code>HGL::Type::RenderCallback</code> (stdlib)
		SERIALID_VIRTUALGROUP, ///< Serial type id of <code>HGL::Type::VirtualGroup</code>
		SERIALID_COMMENT, ///< Serial type id of <code>HGL::Type::Comment</code>
		SERIALID_CURVE, ///< Serial type id of <code>HGL::Type::Curve</code>
		SERIALID_SIGNATURE, ///< Serial type id of <code>HGL::Type::Signature</code>
		SERIALID_INCLUDE, ///< Serial type id of <code>HGL::Type::Include</code>
		SERIALID_CONDITION, ///< Serial type id of <code>HGL::Type::Condition</code>
		SERIALID_IFELSE, ///< Serial type id of <code>HGL::Type::IfElse</code>
		SERIALID_CONDITIONBLOCK, ///< Serial type id of <code>HGL::Type::ConditionBlock</code>
		SERIALID_PEN  ///< Serial type id of <code>HGL::Type::Pen</code>
	} SERIALID;

	inline SERIALID getId() const throw() {
		return m_serialID;
	}

	inline int getWeight() const throw() {
		return m_weight;
	}

	_EXPORT std::string getTypeName() const throw();

	_EXPORT const char *getHumanReadableName() const throw();

	_EXPORT std::string getSourceCodeName() const throw();

	inline bool isParam() const throw() {
		return m_isParam;
	}

	inline bool isReferenceHolder() const throw() {
		return m_isReferenceHolder;
	}

	inline bool isCSPTarget() const throw() {
		return m_isCSPTarget;
	}

	inline bool isDescribeable() const throw() {
		return m_isDescribeable;
	}

	inline bool operator==(const SerialID &o) const throw() {
		return m_serialID == o.m_serialID;
	}

	inline bool operator!=(const SerialID &o) const throw() {
		return !(m_serialID == o.m_serialID);
	}

	_EXPORT static const SerialID &getSerialID(const SERIALID &sid) throw() _PURE;

private:
	SerialID(const SERIALID &serialID, const std::string &tn, const char *const hrn,
			 const std::string &sourceCodeName, bool iParam = false, bool referenceHolder = false,
			 bool cspTarget = false, bool describeable = false, int weight = 0) _NONNULL(4);

public:
	_EXPORT static const uint32_t REFONLY_MAGIC = uint32_t(0x54432654u);

	_EXPORT static const SerialID &END;
	_EXPORT static const SerialID &ANY;
	_EXPORT static const SerialID &NOTSERIALIZEABLE;
	_EXPORT static const SerialID &ASSIGNMENTOPERATOR;
	_EXPORT static const SerialID &COMMENT;
	_EXPORT static const SerialID &CONDITION;
	_EXPORT static const SerialID &CONDITIONBLOCK;
	_EXPORT static const SerialID &CURVE;
	_EXPORT static const SerialID &DEBUGPROCEDURE;
	_EXPORT static const SerialID &ELLIPSE;
	_EXPORT static const SerialID &EXPRESSIONPARAM;
	_EXPORT static const SerialID &FILL;
	_EXPORT static const SerialID &FLOATPARAM;
	_EXPORT static const SerialID &IFELSE;
	_EXPORT static const SerialID &INCLUDE;
	_EXPORT static const SerialID &LINE;
	_EXPORT static const SerialID &MAINSEGMENT;
	_EXPORT static const SerialID &NAMEDPARAM;
	_EXPORT static const SerialID &OBJECTSSEGMENT;
	_EXPORT static const SerialID &PARAMSECTION;
	_EXPORT static const SerialID &PEN;
	_EXPORT static const SerialID &POINT;
	_EXPORT static const SerialID &POLY;
	_EXPORT static const SerialID &PROCCALL;
	_EXPORT static const SerialID &PROCEDURE;
	_EXPORT static const SerialID &PROCEDURESSEGMENT;
	_EXPORT static const SerialID &PROCINITSTATEMENT;
	_EXPORT static const SerialID &PROJECT;
	_EXPORT static const SerialID &REFERENCEPARAM;
	_EXPORT static const SerialID &RENDERCALLBACK;
	_EXPORT static const SerialID &RENDERPROCEDURE;
	_EXPORT static const SerialID &SIGNATURE;
	_EXPORT static const SerialID &SRENDERCALLBACK;
	_EXPORT static const SerialID &STDLIBCALL;
	_EXPORT static const SerialID &STRINGPARAM;
	_EXPORT static const SerialID &TEXT;
	_EXPORT static const SerialID &VECTOR2D;
	_EXPORT static const SerialID &VIRTUALGROUP;

private:
	int m_weight;
	SERIALID m_serialID;
	std::string m_typeName;
	const char *m_humanReadableName;
	std::string m_sourceCodeName;
	bool m_isParam : 1;
	bool m_isReferenceHolder : 1;
	bool m_isCSPTarget : 1;
	bool m_isDescribeable : 1;
};
#pragma GCC diagnostic pop

}

}

#endif /* HGL_TYPE_SERIALID_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
