/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_SINGLETON_H
#define HGL_COMMON_SINGLETON_H

#include <memory>

#include "linkercontrol.h"

namespace HGL {

namespace Common {

/**
 * @ingroup util
 * @brief Template providing the singleton pattern
 *
 * @tparam T class which has to become a singleton
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
template<class T>
class _INTERNAL Singleton {
	Singleton(const Singleton &);
	Singleton &operator=(const Singleton &);

public:
	virtual ~Singleton() = 0;

	/**
	 * @brief Gets the single instance of <code>T</code>
	 *
	 * @return an instance of <code>T</code>
	 */
	static T *getInstance();

protected:
	Singleton() {}

private:
	static std::auto_ptr<T> m_instance;
};

template<class T>
Singleton<T>::~Singleton() {}

template<class T>
T *Singleton<T>::getInstance() {
	if(!m_instance.get()) {
		std::auto_ptr<T> tmp(new T());
		// cppcheck-suppress useAutoPointerCopy
		m_instance = tmp;
	}

	return m_instance.get();
}

template<class T> std::auto_ptr<T> HGL::Common::Singleton<T>::m_instance;

}

}

#endif /* HGL_COMMON_SINGLETON_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
