/*
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *     * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
 *     names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef HGL_COMMON_THREAD_POOL_H
#define HGL_COMMON_THREAD_POOL_H

#include "singleton.h"

namespace HGL {

namespace Common {

/**
 * @ingroup util
 * @brief Template providing a thread pool for asynchronous tasks
 *
 * @tparam Worker pointer to a function actually performing the task
 * @tparam Task a class containing the arguments for the %Worker to do its job
 * @tparam NUM_THREADS maximum number of threads to create
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
template<void (*Worker)(void *), class Task, long NUM_THREADS = 3L>
class _INTERNAL ThreadPool : public Singleton<ThreadPool<Worker, Task, NUM_THREADS> > {
	DISALLOW_COPY_AND_ASSIGN(ThreadPool)

	friend class Singleton<ThreadPool<Worker, Task, NUM_THREADS> >;

	typedef struct {
		pthread_mutex_t job_lock;
		pthread_cond_t  not_empty;
		pthread_cond_t  not_full;
		std::vector<Task *> tasks;
		bool running;
	} work_t;

	inline ThreadPool() : m_tid(), m_isAsync(false), m_isRunning(false) {}

public:
	~ThreadPool();

	/**
	 * @brief Starts the thread pool
	 *
	 * @param num number of threads to start
	 *
	 * @return the number of started threads
	 */
	size_t start(long num = 2L);

	/**
	 * @brief Queue and run a task
	 *
	 * @param task the task to queue and run
	 */
	void run(Task *const task) const _NONNULL(2);

	/**
	 * @brief Run a task asynchrounously
	 *
	 * This method spawns only one thread and behaves basically like calling
	 * @code
	 * start();
	 * run(task);
	 * stop();
	 * @endcode
	 * and stops the thread pool after the task is finished.
	 *
	 * @param task the task to run asynchrounously
	 * @post it is safe to let the calling thread wait()
	 */
	void async(Task *const task) _NONNULL(2);

	/**
	 * @brief Stops the thread pool
	 *
	 * All queued tasks get finished and the thread pool stops
	 *
	 * @post this call is neccessary to let the calling thread wait()
	 */
	void stop();

	/**
	 * @brief Waits for all tasks to get finished
	 *
	 * @pre a call to stop() is neccessary if not waiting for an async job
	 */
	void wait() const;

	/**
	 * @brief Checks if the thread pool is currently running
	 *
	 * @return @c true if the thread pool is currently running, @c false otherwise
	 */
	bool isRunning() const;

private:
	static _NORETURN void *poolWorker(void *work) _NONNULL(1);

private:
	std::vector<pthread_t> m_tid;
	static work_t m_work;
	bool m_isAsync;
	bool m_isRunning;
};

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wold-style-cast"
template<void (*Worker)(void *), class Task, long NUM_THREADS>
typename ThreadPool<Worker, Task, NUM_THREADS>::work_t
ThreadPool<Worker, Task, NUM_THREADS>::m_work = {
	PTHREAD_MUTEX_INITIALIZER,
	PTHREAD_COND_INITIALIZER,
	PTHREAD_COND_INITIALIZER,
	std::vector<Task *>(),
	true
};
#pragma GCC diagnostic pop

template<void (*Worker)(void *), class Task, long NUM_THREADS>
ThreadPool<Worker, Task, NUM_THREADS>::~ThreadPool() {
#ifndef __OpenBSD__

	if(isRunning()) {
		stop();
		wait();
	}

	for(size_t i = 0; i < m_tid.size(); ++i) {
		pthread_detach(m_tid[i]);
	}

#endif
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
void *ThreadPool<Worker, Task, NUM_THREADS>::poolWorker(void *w) {

	work_t *work = reinterpret_cast<work_t *>(w);
	bool stop = false;

	while(!stop) {
		pthread_mutex_lock(&work->job_lock);

		while(work->running && work->tasks.empty()) {
			pthread_cond_wait(&work->not_empty, &work->job_lock);
		}

		Task *task = 0L;

		if(!work->tasks.empty()) {
			task = work->tasks.back();
			work->tasks.pop_back();

			if(work->tasks.size() != work->tasks.max_size()) {
				pthread_cond_broadcast(&work->not_full);
			}
		} else {
			stop = true;
		}

		pthread_mutex_unlock(&work->job_lock);

		if(task) {
			Worker(task);
		}

	}

	pthread_exit(NULL);
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
bool ThreadPool<Worker, Task, NUM_THREADS>::isRunning() const {
#ifndef __OpenBSD__
	return m_isRunning && m_work.running;
#else
	return false;
#endif
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
void ThreadPool<Worker, Task, NUM_THREADS>::async(Task *const task) {
#ifndef __OpenBSD__
	m_isAsync = true;
	start();
	run(task);
	stop();
	m_isAsync = false;
#else

	if(task) {
		Worker(task);
	}

#endif
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
void ThreadPool<Worker, Task, NUM_THREADS>::run(Task *const task) const {
#ifndef __OpenBSD__
	pthread_mutex_lock(&m_work.job_lock);

	while(m_work.tasks.size() == m_work.tasks.max_size()) {
		pthread_cond_wait(&m_work.not_full, &m_work.job_lock);
	}

	if(m_work.tasks.empty()) {
		pthread_cond_signal(&m_work.not_empty);
	}

	if(m_work.running && m_work.tasks.size() != m_work.tasks.max_size()) {
		m_work.tasks.push_back(task);
	}

	pthread_mutex_unlock(&m_work.job_lock);
#else
	const_cast<ThreadPool<Worker, Task, NUM_THREADS> *>(this)->async(task);
#endif
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
size_t ThreadPool<Worker, Task, NUM_THREADS>::start(long num) {
#ifndef __OpenBSD__
	long m_numRunning = m_isAsync ? 1L : std::max(2L, std::min(num, NUM_THREADS));

	wait();

	for(long i = 0; i < m_numRunning; ++i) {
		pthread_t tid;

		if(!pthread_create(&tid, NULL, poolWorker, static_cast<void *>(&m_work))) {
			m_tid.push_back(tid);
		}
	}

	m_isRunning = !m_tid.empty();

	return m_tid.size();
#else
	return 0;
#endif
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
void ThreadPool<Worker, Task, NUM_THREADS>::stop() {
#ifndef __OpenBSD__
	pthread_mutex_lock(&m_work.job_lock);
	m_work.running = false;

	pthread_cond_broadcast(&m_work.not_empty);
	pthread_cond_broadcast(&m_work.not_full);

	pthread_mutex_unlock(&m_work.job_lock);

	m_isRunning = false;
#endif
}

template<void (*Worker)(void *), class Task, long NUM_THREADS>
void ThreadPool<Worker, Task, NUM_THREADS>::wait() const {
#ifndef __OpenBSD__

	for(size_t i = 0; i < m_tid.size(); ++i) {
		pthread_join(m_tid[i], NULL);
	}

#endif
}

}

}

#endif /* HGL_COMMON_THREAD_POOL_H */

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
