/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_TYPEBASE_H
#define HGL_TYPE_TYPEBASE_H

extern "C" {
#include <lua.h>
}

#include "itype.h"

namespace Commons {
template<class> class BasicLogger;
}

namespace HGL {

namespace Type {

namespace Utility {
template<class> class EqualTypeVisitor;
}

class TypeBaseImpl;

/**
 * @ingroup types
 * @brief This class provides the default implementation for most types
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT TypeBase : public virtual IType {

	TypeBase &operator=(const TypeBase &);

	template<class T> friend HGL::IType::Ptr::operator T *() const throw();
	template<class> friend class HGL::Type::Utility::EqualTypeVisitor;
	friend class TypeBaseImpl;
	friend class ExpressionParamImpl;

	friend int getProjectName(lua_State *);
	friend int listTypes(lua_State *);

public:
	using IType::operator*;
	using IType::attachType;

	virtual const std::string &getId() const _HOT _PURE;
	virtual void setId(const std::string &id);

	virtual void setDescription(const std::string &desc)
	throw(Exception::InvalidExpressionException);

	IType *attachType(IType *type, bool check = true) throw(Exception::AttachException) _HOT;
	virtual void detachType(IType *type);
	void detachAll() const;

	virtual const TYPELIST &getAttachedTypes() const throw() _HOT _PURE;

	virtual IType::iterator begin();
	virtual IType::iterator end();

	virtual bool isNull() const throw() _CONST;

	virtual IType *getParent() const _PURE;
	virtual void setRoot(IType *);

	virtual uint32_t getControlSourceParameter() const _PURE;

	virtual bool operator==(const IType &) const throw(Exception::InvalidExpressionException);
	virtual bool operator<(const IType &) const throw(Exception::InvalidExpressionException);

	virtual IType *operator+(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual IType *operator-(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual IType *operator*(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual IType *operator*(const float *) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException) _NONNULL(2);
	virtual IType *operator/(const IType &) throw(Exception::ReferenceException,
			Exception::InvalidExpressionException);
	virtual IType *operator-() throw(Exception::InvalidExpressionException);

	virtual IType *power(const IType &) throw(Exception::InvalidExpressionException);
	virtual IType *sinus() throw(Exception::InvalidExpressionException);
	virtual IType *cosinus() throw(Exception::InvalidExpressionException);

	virtual BOUNDINGBOX getBoundingBox() const;

protected:
	TypeBase();
	TypeBase(const TypeBase &);
	explicit TypeBase(IType *const parent);
	explicit TypeBase(const std::string &id);
	TypeBase(const std::string &id, const std::string &dsc);
	virtual ~TypeBase();

	virtual operator Type::Vector2D &() const throw(Exception::InvalidExpressionException);
	virtual operator Type::StringParam &() const throw(Exception::InvalidExpressionException);

	_NOUNUSED virtual IType::Ptr clone() const throw(Exception::CloneNotSupportedException);

	/**
	 * @brief Displayable name of the IType
	 * @see SerialID::getTypeName()
	 * @return the displayable name of the IType
	 **/
	virtual std::string getTypeName() const throw();

	virtual void setParent(IType *parent);

	virtual const IType *getRoot() const _PURE;
	virtual const IType *getRoot(const IType &type) const _HOT;

	/**
	 * @brief Attaches a child at the position @c before
	 *
	 * It updates the parent of the attached type to this type as well.
	 *
	 * @param type the IType to attach as child
	 * @param before the type will be attached @b before the type referenced by @c before
	 * @param check @c true (default), if validity checks should be performed
	 * @copydetails IType::attachType
	 **/
	virtual IType *attachType(IType *type, const TYPELIST::iterator &before,
							  bool check = true) throw(Exception::AttachException) _NONNULL(2) _HOT;

	TYPELIST *getAttachedTypes() throw() _HOT;

	virtual void invalidateIterator(bool parent = false) const;

	bool isAllowedToAttach(const IType *const) const throw() _NONNULL_ALL;

	virtual operator float() const throw(Exception::ReferenceException,
										 Exception::InvalidExpressionException);

	virtual IType &operator=(const IType &) throw(Exception::InvalidExpressionException);

	/**
	 * @details A type is @b valid<ol>
	 * <li>if all attached types are allowed for this type, if not the type is @b invalid </li>
	 * <li>if all attached types itself are valid, if not the type is @b invalid </li></ol>
	 *
	 * Subclasses can add more checks if neccessary.
	 *
	 * @copydoc IType::isValid
	 **/
	virtual bool isValid() const throw();

	virtual const std::string &getDescription() const _PURE;
	virtual void setControlSourceParameter(uint32_t par)
	throw(Exception::InvalidExpressionException);

	_NOUNUSED virtual _HOT IType *resolve(const std::string &id) const;

private:
	virtual const IType &operator->() const {
		return *this;
	}

	virtual operator IType::iterator();
	virtual operator IType::iterator &();

	_INTERNAL virtual const Type::SerialID *const *getAllowedTypes() const throw() = 0;

	static IType *getProjectRoot() _PURE;

private:
	TypeBaseImpl *const _pimpl;
};

}

_EXPORT extern Commons::BasicLogger<std::ostreambuf_iterator<wchar_t> >
&operator<<(Commons::BasicLogger<std::ostreambuf_iterator<wchar_t> > &,
			const HGL::IType::BOUNDINGBOX &);

}

#endif // HGL_TYPE_TYPEBASE_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
