/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_TYPE_UTIL_H
#define HGL_TYPE_UTIL_H

#if defined(HAVE_CONFIG_H) || defined(IN_KDEVELOP_PARSER)
#include "config.h"
#endif

#include <set>

#if ENABLE_SIGNING
#include <gpgme.h>
#endif

#include "itype.h"
#include "util.h"

namespace HGL {

namespace Common {
class RegexInitializer;
}

namespace Type {

/**
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _PACKED Util : public Common::Util {
	DISALLOW_COPY_AND_ASSIGN(Util)
public:

#if ENABLE_SIGNING
	inline static std::string getGPGMEKeyInfo(const gpgme_key_t key) {
		return format("%s (%s <%s>) const", &key->subkeys->_keyid[8], key->uids->name, key->uids->email);
	}
#endif

	_EXPORT static bool checkPenDescription(const char *const penDesc);
	_EXPORT static std::string compactPenDescription(const char *const penDesc);

	inline static std::string colorInteger2Hex(uint32_t intCol) {
		return format("%.8X", intCol);
	}

	/**
	 * @brief Converts a hex color string or (optionally) a color name into an integer
	 *
	 * Preset parameter @c ok with a pointer pointing to @c true to enable a color name lookup
	 *
	 * @param hex hex color string or (optionally) a color name
	 * @param ok if not @c NULL than store @c true if conversion succeeded
	 *
	 * @return an 32-bit integer representing the resulting color
	 */
	_EXPORT static uint32_t colorHex2Integer(const std::string &hex, bool *ok = 0L);

	inline static std::string generateSessionUniqueId(const std::string &aid,
			unsigned long long start = 0u) {
		return format("%s%.1llX", aid.c_str(), getNxt(start));
	}

	typedef void (*AIDMARKER)(HGL::IType *);

	_EXPORT static std::string sanitizeId(const std::string &id);
	_EXPORT static std::string createNamespace(HGL::IType *type, bool autoID = false,
			const std::string &aid = std::string(), unsigned long long start = 0u,
			AIDMARKER aidmarkerCallback = 0L, std::set<HGL::IType *> *const path = 0L) throw();

	_EXPORT static HGL::IType::TYPELIST::const_iterator begin(const IType *type) _HOT;
	_EXPORT static HGL::IType::TYPELIST::const_iterator end(const IType *type) _HOT;

	template<class T>
	_EXPORT static T gcd(const T &a, const T &b);

protected:
	inline Util() {};

	inline static unsigned long long getNxt(unsigned long long start = 0u) {
		return start + (m_nxt++);
	}

private:
	static Common::RegexInitializer m_hexColRex;
	static Common::RegexInitializer m_penDscRex;
	static unsigned long long m_nxt _EXPORT;
};

template<class T>
_CONST inline T Util::gcd(const T &a, const T &b) {

	T u = a;
	T v = b;

	while(v != 0)  {
		T t = v;
		v = u % v;
		u = t;
	}

	return u;
}

}

}

extern template class std::set<HGL::IType *>;

#endif // HGL_TYPE_UTIL_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
