/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_COMMON_URI_H
#define HGL_COMMON_URI_H

#include <istream>
#include <set>

#include "ioexception.h"

#if WIN32
#undef IN
#endif

namespace HGL {

namespace Common {

class URIImpl;

/**
 * @ingroup util
 * @brief Class to access files locally and remote
 *
 * This class handles the access of local files as well as remote files and from the
 * standard input.
 *
 * Since version <em>0.5.18</em> it implements std::istream and therefore can be used as
 * input stream as well.
 *
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _EXPORT URI : public std::istream {

	typedef enum { EXT, IN, OUT } STD;
	explicit URI(const STD &);
	friend class URIImpl;

	URI &operator=(const URI &);

public:
	_INTERNAL URI(const URI &);

	/**
	 * @brief @c Constructor
	 *
	 * Creates an @c URI with open mode <em>@c "r" </em>
	 *
	 * @param uri URI to open
	 */
	URI(const char *const uri) throw(HGL::Exception::IOException);

	/**
	 * @brief @c Constructor
	 *
	 * @param uri URI to open
	 * @param mode open mode, i.e. <em>@c "r" </em> or <em>@c "rb" </em>
	 */
	URI(const std::string &uri, const char *const mode)
	throw(HGL::Exception::IOException) _NONNULL(3);

	/**
	 * @brief @c Constructor
	 *
	 * @param uri URI to open
	 * @param mode open mode, i.e. <em>@c "r" </em> or <em>@c "rb" </em>
	 */
	_INTERNAL URI(const char *const uri, const char *const mode)
	throw(HGL::Exception::IOException) _NONNULL(3);

	~URI();

	/**
	 * @brief Set of supported protocols
	 **/
	typedef std::set<std::string> PROTOCOLS;

	/**
	 * @brief Checks if this @c URI is attached to standard input
	 *
	 * @return @c true if attached to standard input, @c false otherwise
	 * @since 0.5.18
	 **/
	bool isStdin() const _PURE;

	/**
	 * @brief Gets the URI
	 *
	 * @return the URI
	 */
	std::string getURI() const;

	/**
	 * @brief Gets the protocol
	 *
	 * @return the protocol
	 */
	std::string getProtocol() const;

	/**
	 * @brief Gets the size in bytes of the file
	 *
	 * @return the size in bytes
	 */
#if defined __x86_64__ || !defined __USE_FILE_OFFSET64
	__off_t
#else
	__off64_t
#endif
	getSize() const throw(HGL::Exception::IOException);

	/**
	 * @brief Gets the @c FILE file descriptor of the file
	 *
	 * @return the @c FILE file descriptor
	 */
	operator FILE *() const throw(HGL::Exception::IOException);

	/**
	 * @brief Reads the entire file into a @c std::string
	 *
	 * @param s the @c std::string to read the entire file in
	 */
	_INTERNAL void operator>>(std::string &s) const throw(HGL::Exception::IOException);

	/**
	 * @brief Gets a set of all supported protocols
	 *
	 * @return a set of all supported protocols
	 */
	_INTERNAL static const PROTOCOLS getProtocols();

	/**
	 * @brief Checks if the URI is a remote URI
	 *
	 * @param uri the uri to check
	 * @return @c true, if the URI is remote, @c false otherwise
	 */
	static bool isRemote(const std::string &uri);

	/**
	 * @brief Pushes a path to the directories to prepend if the file is not found
	 *
	 * @note this works only for relative paths
	 *
	 * @param searchPath the path to push to the serach patch list
	 * @param front if @c true, the path gets prepended as first search element
	 */
	_INTERNAL static void pushSearchPath(const std::string &searchPath, bool front = false);

	/**
	 * @brief Tries to prefetch a remote URL in background
	 *
	 * If the file could be prefetches successfully, subsequent access to it won't be downloaded
	 * again, but the locally downloaded copy is accessed.
	 *
	 * @param uri the URI to prefetch
	 * @param mode open mode, i.e. <em>@c "r" </em> or <em>@c "rb" </em>
	 */
	static void prefetch(const std::string &uri, const char *const mode) _NONNULL(2);

private:
	URI();
	void init() throw(Exception::IOException);

public:
	_EXPORT static URI STDIN; ///< an URI which points to @c stdin

private:
	class URIImpl *const _pimpl;
};

}

}

#endif // HGL_COMMON_URI_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
