/*
    Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of Heiko Schäfer <heiko@hgl.rangun.de> nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de> BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HGL_COMMON_UTIL_H
#define HGL_COMMON_UTIL_H

#include <vector>
#include <fstream>
#include <cstdarg>
#include <algorithm>

#include <iconv.h>

#include "eff_map.h"
#include "floatutil.h"
#include "ioexception.h"
#include "encodingexception.h"
#include "terminateexception.h"

/**
 * @defgroup util Utility classes
 * @brief Classes and methods providing commonly used utilites
 **/

extern template class std::vector<std::string>;

_EXPORT extern float atof_nol(const char *const);
_EXPORT extern float atof_nol(const std::string &);

namespace HGL {

namespace Common {

class IApplication;

/**
 * @ingroup util
 * @brief Miscellaneous utility functions
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 **/
class _PACKED Util : public FloatUtil {
	DISALLOW_COPY_AND_ASSIGN(Util)
public:
	/**
	 * @brief Logs the amount of WARNINGs already issued
	 *
	 * @param app if @c NULL (default) log as warning, else as notification
	 */
	_EXPORT static void logWarningCount(const IApplication *const app = 0L);

	/**
	 * @brief Appends a suffix to a filename
	 *
	 * The @c suffix is appended to @c ofile if it hasn't already that suffix.
	 * The comparision if @c ofile already has @c suffix appended is done case insensitively.
	 *
	 * @param[in] suffix the suffix to append
	 * @param[in,out] oFile the filename to append @c suffix to
	 * @return @c oFile with @c suffix appended
	 *
	 * @since 0.5.18
	 */
	_EXPORT static std::string &suffixAppender(const char *const suffix,
			std::string &oFile) _NONNULL(1);

	/**
	 * @brief Gets a value from the configuration file
	 *
	 * @param key the key to search the config value for
	 *
	 * @return a pointer to the value or @c NULL if there is no such value for @c key
	 */
	_EXPORT _NOUNUSED static std::string const *getConfigValue(const std::string &key) _PURE;

	_EXPORT static bool notificationsEnabled();

	_EXPORT static std::wstring
	convertFromUTF8(const char *const utf8) throw(HGL::Exception::EncodingException);

	_EXPORT static std::string
	convertToUTF8(const wchar_t *wcs) throw(HGL::Exception::EncodingException);

	template<typename OSeq, typename ISeqIter, typename Operation>
	inline static OSeq sequenceTransform(const ISeqIter &first, const ISeqIter &last,
										 Operation op) {
		OSeq result;
		return sequenceTransform(first, last, result, op);
	}

	template<typename ISeqIter, typename OSeq, typename Operation>
	static OSeq &sequenceTransform(const ISeqIter &first, const ISeqIter &last, OSeq &result,
								   Operation op) {
		result.reserve(result.size() + std::distance(first, last));
		std::transform(first, last, std::back_inserter(result), op);
		return result;
	}

	_EXPORT static std::string format(const char *const fmt, ...)
	__attribute__((format(printf, 1, 2)));
	_EXPORT static std::string vformat(const char *const fmt, std::va_list ap);
	_EXPORT static std::string trim(const std::string &s);
	_EXPORT static bool cmpi(const std::string &l, const std::string &r);
	_EXPORT static std::vector<std::string> split(const std::string &source,
			const char *const delimiter = " ", bool keepEmpty = false) _NONNULL(2);

	inline static void disableMemoryHandler() {
		m_terminateHandler.disableMemoryHandler();
	}

	inline static void enableMemoryHandler() {
		m_terminateHandler.enableMemoryHandler();
	}

	typedef std::map<std::string, std::string> COLORNAMEMAP;
	_EXPORT static const COLORNAMEMAP &getColorNameMap();
	_EXPORT static const COLORNAMEMAP &getReverseColorNameMap() _CONST;

	template<typename T>
	inline static void nop1(T) {}

	inline static void nop2() {}

	template<class Container>
	inline static void clear_and_minimize(Container &c) {
		using std::swap;
		Container().swap(c);
	}

	template<class Container>
	static void shrink_to_fit(Container &c);

	template<class Stream>
	static Stream *checkOpen(Stream *str, const char *const fname) throw(Exception::IOException);

protected:
	Util();

	static int keyValLexer(FILE *io);
	_EXPORT static bool configValueToBool(const std::string &key, bool defaultIfUnset = false);

private:
	_EXPORT static std::string openFailMsg(const std::ios_base::openmode &om,
										   const char *const fname);
	static std::map<std::string, std::string> generateRevColMap();

private:
#ifndef WIN32
	static _WEAK class _PACKED _terminateHandler {
#else
	static class _PACKED _terminateHandler {
#endif
		DISALLOW_COPY_AND_ASSIGN(_terminateHandler)
	public:
		_terminateHandler() _COLD;
		inline ~_terminateHandler() _COLD {}

		_EXPORT void disableMemoryHandler() _HOT;
		_EXPORT void enableMemoryHandler() _HOT;

	private:
		static void outOfMemory() _COLD;
		static void unexpected() throw(HGL::Exception::TerminateException) _COLD;
		static void terminate() _COLD;

	private:
		std::new_handler m_origHandler;

		bool m_oomDisabled;

		static pthread_mutex_t m_lock;
		static pthread_cond_t  m_cond;
	} m_terminateHandler _EXPORT;

	static class _PACKED _initIconv {
		DISALLOW_COPY_AND_ASSIGN(_initIconv)
	public:
		_initIconv(const char *const to, const char *const from)
		throw(HGL::Exception::EncodingException) _NONNULL(2, 3) _COLD;
		~_initIconv() _COLD;
		iconv_t m_iconv;
	} m_initIconvFromUTF8, m_initIconvToUTF8;

#ifndef WIN32
	static _WEAK class _configReader {
		DISALLOW_COPY_AND_ASSIGN(_configReader)
	public:
		_EXPORT _configReader(const std::string &name) _COLD;
		_EXPORT ~_configReader() _COLD;

		std::map<std::string, std::string> m_configMap;

	private:
		_INTERNAL void mergeConfig(const std::string &path) _COLD;
	} m_config _EXPORT;
#endif

	static COLORNAMEMAP m_colorNames;
	static COLORNAMEMAP m_revColMap;
};

template<class Container>
void Util::shrink_to_fit(Container &c) {
	using std::swap;
	Container(c).swap(c);
}

template<> inline std::ofstream *Util::checkOpen<std::ofstream>(std::ofstream *str,
		const char *const fname) throw(Exception::IOException) {

	if(str && !str->is_open()) {
		throw HGL::Exception::IOException(openFailMsg(std::ios_base::out, fname));
	}

	return str;
}

template<> inline std::ifstream *Util::checkOpen<std::ifstream>(std::ifstream *str,
		const char *const fname) throw(Exception::IOException) {

	if(str && !str->is_open()) {
		throw HGL::Exception::IOException(openFailMsg(std::ios_base::in, fname));
	}

	return str;
}

/**
 * @ingroup util
 * @brief Copies @c src into @c dst while the predicate @c pred is @c true
 *
 * @tparam InputIterator  an @c InputIterator
 * @tparam OutputIterator an @c OutputIterator
 * @tparam UnaryPredicate a predicate
 *
 * @param src  the @c InputIterator
 * @param dst  the @c OutputIterator
 * @param pred the predicate
 *
 * @since 0.5.26
 */
template<class InputIterator, class OutputIterator, class UnaryPredicate>
static inline OutputIterator copy_while(InputIterator src, OutputIterator dst, UnaryPredicate pred) {

	__glibcxx_function_requires(_InputIteratorConcept<InputIterator>)
	__glibcxx_function_requires(_OutputIteratorConcept < OutputIterator,
								typename std::iterator_traits<InputIterator>::value_type >)
	__glibcxx_function_requires(_UnaryPredicateConcept < UnaryPredicate,
								typename std::iterator_traits<InputIterator>::value_type >)

	while(pred(*src)) {
		*dst = *src;
		++dst;
		++src;
	}

	return dst;
}

/**
 * @ingroup util
 * @brief Copies @c src into @c dst until the predicate @c pred is @c true
 *
 * @tparam InputIterator  an @c InputIterator
 * @tparam OutputIterator an @c OutputIterator
 * @tparam UnaryPredicate a predicate
 *
 * @param src  the @c InputIterator
 * @param dst  the @c OutputIterator
 * @param pred the predicate
 *
 * @since 0.5.26
 */
template<class InputIterator, class OutputIterator, class UnaryPredicate>
OutputIterator copy_until(InputIterator src, OutputIterator dst, UnaryPredicate pred) {
	return copy_while(src, dst, std::not1(pred));
}

}

}

extern template class std::basic_string<char>;
extern template class std::map<std::string, std::string>;

#endif // HGL_COMMON_UTIL_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
