/***
 * Copyright (c) 2012-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

#ifndef HGL_VECTOR2D_H
#define HGL_VECTOR2D_H

#include "parambase.h"

#ifdef CPPUNIT
class LuaProcedure;
#endif

namespace HGL {

namespace ControlSource {
class ParameterFactory;
}

namespace Type {

/**
 * @ingroup types
 * @ingroup params
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 */
class _EXPORT _PACKED Vector2D : public ParamBase, public BaseVector2D {
	Vector2D(const Vector2D &);

	template<class T> friend IType::Ptr::operator T *() const throw();

	friend class HGL::ControlSource::ParameterFactory;
	friend struct IType::_boundingbox;
	friend class ProjectFactoryBase;
	friend class FloatParam;
	friend class Line;

#ifdef CPPUNIT
	friend class ::LuaProcedure;
#endif

public:
	ACCEPT(Vector2D);

	~Vector2D();

	virtual const std::string toString() const;

	virtual operator StringParam &() const throw(Exception::InvalidExpressionException);
	virtual operator Vector2D &() const throw(Exception::InvalidExpressionException) _CONST;

	Vector2D &operator=(const Vector2D &op) throw(Exception::InvalidExpressionException);
	virtual Vector2D &operator=(const IType &op) throw(Exception::InvalidExpressionException);

	virtual IType *operator+(const IType &op) throw(Exception::InvalidExpressionException);
	virtual IType *operator-(const IType &op) throw(Exception::InvalidExpressionException);
	virtual IType *operator*(const IType &op) throw(Exception::InvalidExpressionException);
	virtual IType *operator*(const float *) throw(Exception::InvalidExpressionException) _NONNULL(2);
	virtual IType *operator/(const IType &op) throw(Exception::InvalidExpressionException);
	virtual Vector2D *operator-() throw(Exception::InvalidExpressionException);

	virtual IType *power(const IType &op) throw(Exception::InvalidExpressionException);

	virtual BOUNDINGBOX getBoundingBox() const;

protected:
	Vector2D(float x, float y);

	const SerialID &getSerialID() const throw() _HOT _CONST;

	virtual void setControlSourceParameter(uint32_t par)
	throw(Exception::InvalidExpressionException);

private:
	_INTERNAL Vector2D();

	template<class Operator, class Fallback>
	_INTERNAL IType *operate(const IType &operand, Operator op,
							 Fallback fallback) throw(Exception::InvalidExpressionException);
};

template<class Operator, class Fallback>
HGL::IType *Vector2D::operate(const IType &operand, Operator op,
							  Fallback fallback) throw(Exception::InvalidExpressionException) {

	try {
		const Vector2D &v(operand);

		m_x = op(m_x, v.m_x);
		m_y = op(m_y, v.m_y);

		return this;

	} catch(const Exception::InvalidExpressionException &) {
		return fallback(this, operand);
	}
}

extern template HGL::IType *HGL::Type::Vector2D::operate < float( *)(float, float),
	   HGL::IType *( *)(HGL::Type::Vector2D *, HGL::IType const &) > (HGL::IType const &,
			   float( *)(float, float), HGL::IType * ( *)(HGL::Type::Vector2D *,
					   HGL::IType const &));

}

template<> inline _HOT IType::Ptr::operator Type::Vector2D *() const throw() {
	return reinterpret_cast<Type::Vector2D *>(m_ptr);
}

extern template IType::Ptr::operator Type::Vector2D *() const throw();

}

#endif

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
