/***
 * Copyright (c) 2013-2014, Heiko Schäfer <heiko@hgl.rangun.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this list of conditions and
 * the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 * and the following disclaimer in the documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Heiko Schäfer <heiko@hgl.rangun.de> "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Heiko Schäfer <heiko@hgl.rangun.de>
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

/**
 * @file writer.h
 * @brief A simple exception safe writer for image output plugins
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 **/

#ifndef HGL_IMAGEOUTPUT_WRITER_H
#define HGL_IMAGEOUTPUT_WRITER_H

#include "imageoutputexception.h"

namespace HGL {

namespace ImageOutput {

/**
 * @ingroup ioplugins
 * @brief A simple exception safe writer for image output plugins
 * @author Heiko Schäfer <heiko@hgl.rangun.de>
 * @copyright BSD License
 * @since 0.4.52
 **/
class _EXPORT Writer {
	Writer(const Writer &);
	Writer &operator=(const Writer &);

public:
	/**
	 * @brief Constructs an <code>Writer</code> and initializes it for use
	 *
	 * This writer opens the file or throws an ImageOutputException if it failed.
	 *
	 * @throw ImageOutputException if the file could not get created
	 *
	 * @param filename the filename or an empty string for output to stdout
	 * @param suffix appends @c suffix if not @c NULL and not given
	 *
	 * @since 0.4.52
	 **/
	Writer(const std::string &filename, const char *const suffix = 0L) throw(ImageOutputException);

	/**
	 * @brief Closes the file
	 *
	 * If no call to Writer::endWrite was done, it will remove the file.
	 *
	 * @since 0.4.52
	 **/
	~Writer();

	/**
	 * @brief Gets the file handle of the file
	 *
	 * @return int the file handle
	 * @since 0.4.52
	 **/
	int getHandle() const;

	/**
	 * @brief Gets a pointer to the FILE structure of the file
	 *
	 * @return a pointer to the FILE structure of the file
	 * @since 0.4.52
	 **/
	inline FILE *getFile() const {
		return m_file;
	}

	/**
	 * @brief Gets a pointer to the FILE structure of the file
	 *
	 * @return a pointer to the FILE structure of the file
	 * @since 0.5.4
	 **/
	inline operator FILE *() const {
		return m_file;
	}

	/**
	 * @brief Writes a string into the file
	 *
	 * @param s the string to write
	 * @return reference to the Writer to concatenate this operator
	 * @since 0.4.70
	 **/
	Writer &operator<<(const std::string &s) throw(ImageOutputException);

	/**
	 * @brief Writes a float into the file
	 *
	 * @param f the float to write
	 * @return reference to the Writer to concatenate this operator
	 * @since 0.5.2
	 **/
	Writer &operator<<(float f) throw(ImageOutputException);

	/**
	 * @brief Indicates a successful end of write of the file
	 *
	 * If no call to this function is done before <code>Writer</code> gets destroyed
	 * it will delete the file.
	 *
	 * @since 0.4.52
	 **/
	inline void endWrite() {
		endWrite(false);
	}

	/**
	 * @brief Indicates a successful end of write of the file
	 *
	 * If no call to this function is done before <code>Writer</code> gets destroyed
	 * it will delete the file.
	 *
	 * @param noClose @c true if the file needs not to get closed, @c false otherwise
	 *
	 * @since 0.5.17
	 **/
	inline void endWrite(bool noClose) {
		m_needsClose = !noClose;
		m_writeFinished = true;
	}

	inline std::string getFilename() const {
		return m_filename;
	}

private:
	FILE *m_file;
	std::string m_filename;

	bool m_needsClose : 1;
	bool m_writeFinished : 1;
};

}

}

#endif // HGL_IMAGEOUTPUT_WRITER_H

// kate: indent-mode cstyle; indent-width 4; replace-tabs off; tab-width 4; 
